import java.awt.*;

public class Abc extends Symbol {
  // align = -1(left) 0(center) 1(right)
  // valign = -1(down) 0(center) 1(up)
  int align;
  int valign;
  int size;
  int style;
  String caption;
  String type;
  Font font;
  FontMetrics metrics;

  // ask FontList class
  boolean fListCreate = true;
  FontList fontList;
  String fontMatch;

  // constants
  final int DEFAULT_ALIGN = -1;  // left
  final int DEFAULT_VALIGN = -1; // down
  final int DEFAULT_SIZE = 12;
  final int DEFAULT_STYLE = Font.PLAIN;
  String DEFAULT_TYPE = "Dialog";
  final Color DEFAULT_COLOR = Color.black;

  void initFontList()
  {
    if(fListCreate == true) {
      fontList = new FontList();
      fListCreate = false;
    }
  }

  Abc()
  {
    initFontList();
    fontMatch = fontList.get(DEFAULT_TYPE);

    // default
    this.align = DEFAULT_ALIGN;
    this.valign = DEFAULT_VALIGN;
    this.font = new Font(fontMatch, DEFAULT_STYLE, DEFAULT_SIZE);
    this.metrics = getFontMetrics(font);
    this.color = DEFAULT_COLOR;
    this.caption = null;
    this.type = DEFAULT_TYPE;
  }

  Abc(String caption, Point position)
  {
    initFontList();
    fontMatch = fontList.get(DEFAULT_TYPE);

    this.position = position;
    this.caption = caption;

    // default
    this.align = DEFAULT_ALIGN;
    this.valign = DEFAULT_VALIGN;
    this.font = new Font(fontMatch, DEFAULT_STYLE, DEFAULT_SIZE);
    this.metrics = getFontMetrics(font);
    this.color = DEFAULT_COLOR;
    this.type = DEFAULT_TYPE;
  }

  Abc(String caption, int size)
  {
    initFontList();
    fontMatch = fontList.get(DEFAULT_TYPE);

    this.caption = caption;
    this.size = size;

    // default
    this.align = DEFAULT_ALIGN;
    this.valign = DEFAULT_VALIGN;
    this.font = new Font(fontMatch, DEFAULT_STYLE, size);
    this.metrics = getFontMetrics(this.font);
    this.color = DEFAULT_COLOR;
    this.type = DEFAULT_TYPE;
  }

  Abc(String caption, Point position, int size)
  {
    initFontList();
    fontMatch = fontList.get(DEFAULT_TYPE);

    this.caption = caption;
    this.position = position;
    this.size = size;

    // default
    this.align = DEFAULT_ALIGN;
    this.valign = DEFAULT_VALIGN;
    this.font = new Font(fontMatch, DEFAULT_STYLE, size);
    this.metrics = getFontMetrics(this.font);
    this.color = DEFAULT_COLOR;
    this.type = DEFAULT_TYPE;
  }

  Abc(String caption, Color color)
  {
    initFontList();
    fontMatch = fontList.get(DEFAULT_TYPE);

    this.caption = caption;
    this.color = color;

    // default
    this.align = DEFAULT_ALIGN;
    this.valign = DEFAULT_VALIGN;
    this.font = new Font(fontMatch, DEFAULT_STYLE, DEFAULT_SIZE);
    this.metrics = getFontMetrics(this.font);
    this.color = DEFAULT_COLOR;
    this.type = DEFAULT_TYPE;
  }

  Abc(String caption, int size, int style, String type)
  {
    initFontList();
    fontMatch = fontList.get(type);

    this.caption = caption;
    this.size = size;
    this.style = style;
    this.type = type;

    // default
    this.align = DEFAULT_ALIGN;
    this.valign = DEFAULT_VALIGN;
    this.font = new Font(fontMatch, style, size);
    this.metrics = getFontMetrics(this.font);
    this.color = DEFAULT_COLOR;
    this.type = DEFAULT_TYPE;
  }

  Abc(String caption, int size, int style)
  {
    initFontList();
    fontMatch = fontList.get(DEFAULT_TYPE);

    this.caption = caption;
    this.size = size;
    this.style = style;

    // default
    this.align = DEFAULT_ALIGN;
    this.valign = DEFAULT_VALIGN;
    this.font = new Font(fontMatch, style, size);
    this.metrics = getFontMetrics(this.font);
    this.color = DEFAULT_COLOR;
    this.type = DEFAULT_TYPE;
  }

  Abc(Point position, int align, int valign, String caption, int size,
      int style, Color color)
  {
    initFontList();
    fontMatch = fontList.get(DEFAULT_TYPE);

    this.position = position;
    this.align = align;
    this.valign = valign;
    this.font = new Font(fontMatch, style, size);
    this.metrics = getFontMetrics(this.font);
    this.color = color;
    this.caption = caption;
    this.type = DEFAULT_TYPE;
  }

  void setColor(Color color)
  {
    this.color = color;
  }

  void setStyle(int style)
  {
    initFontList();
    fontMatch = fontList.get(this.type);

    this.style = style;
    this.font = new Font(fontMatch, style, this.size);
    this.metrics = getFontMetrics(font);
  }

  void setType(String type)
  {
    initFontList();
    fontMatch = fontList.get(type);

    this.type = type;
    this.font = new Font(fontMatch, this.style, this.size);
    this.metrics = getFontMetrics(font);
  }

  void setPosition(Point position)
  {
    this.position = position;
  }

  void setAlign(int align, int valign)
  {
    this.align = align;
    this.valign = valign;
  }

  void setCaption(String caption)
  {
    this.caption = caption;
  }

  void draw(Graphics g)
  {
    Color currentColor;
    Font currentFont;
    Point baseline = new Point(position.x, position.y);

    // horizontal alignment
    switch(align) {
      case  0: // center
               baseline.x -= metrics.stringWidth(caption)/2; 
               break;
      case  1: // right
               baseline.x -= metrics.stringWidth(caption); 
               break;
      default: // left
               break;
    }

    // vertical alignment
    switch(valign) {
      case  0: // center
               //baseline.y += metrics.getHeight()/4; 
               baseline.y += (ascent() - descent())/2;
               break;
      case  1: // up
               //baseline.y += metrics.getHeight()/2; 
               baseline.y += (ascent() - descent());
               break;
      default: // down
               break;
    }

    currentColor = g.getColor();
    currentFont = g.getFont();
    g.setFont(font);
    g.setColor(color);
    g.drawString(caption, baseline.x, baseline.y);

    // restore
    g.setColor(currentColor);
    g.setFont(currentFont);
  } 

  int width() 
  {
    return(metrics.stringWidth(caption)); 
  }

  int height()
  {
    return(metrics.getHeight()); 
  }

  int ascent()
  {
    return(metrics.getAscent()); 
  }

  int descent()
  {
    return(metrics.getDescent()); 
  }

  int leading()
  {
    return(metrics.getLeading()); 
  }
}
