import java.awt.*;

class VisualLegend {
  Point position;        // lower left
  Abc caption;
  LegendMark mark;

  final int CAPTION_DX = 10;    // x spacing between mark and caption 
                                // (bounding boxes) 

  VisualLegend(int x, int y, LegendMark mark, Abc caption) 
  {
    position = new Point(x,y);  // (left, center)

    // create elements
    this.caption = caption;
    this.mark = mark;
    
    updatePositions();
  }

  VisualLegend(LegendMark mark, Abc caption) 
  {
    // create elements
    this.caption = caption;
    this.mark = mark;
  }

  void setPosition(int x, int y)
  {
    position = new Point(x,y);
    updatePositions();
  }

  void updatePositions()
  {
    Point pos;

    // set mark position
    pos = new Point(position.x, 
                    position.y - mark.height()/2);
    mark.setPosition(pos);

    // set caption position with displacement in x
    pos = new Point(position.x + mark.width() + CAPTION_DX, 
                    position.y + mark.height()/2);
    caption.setPosition(pos);
  }

  int height()
  {
    return(Math.max(mark.height(), caption.height())); 
  }

  int width()
  {
    return(mark.width() + caption.width() + CAPTION_DX); 
  }

  void draw(Graphics g)
  {
    mark.draw(g);
    caption.draw(g);
  }
}
