import java.net.URL;
import java.net.Socket;
import java.util.Date;
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.servlet.http.*;

public class AgentGifEncoder implements AgentInterface
{
  // dynamic parameters: may change every call
  String data;
  int width;
  int height;

  // static parameters: read only the 1st time 
  static String wwwRootDir = null;
  static String baseDir = null;
  static String host = null;
  static int wakeupTime;                   // in minutes

  static TmpFilenameGenerator fileGen;
  //static TmpFileCollector fileCollector;
  static boolean init = false;

  ObjectInputStream inStream = null;
  ObjectOutputStream outStream = null;

  static final String DEFAULT_IMAGE_DIRECTORY = "images";

  public void run(Socket client)
  {
    try {
    	System.out.println("Antes de ler o input stream fornecido pelo cliente");
      inStream = new ObjectInputStream(client.getInputStream());
      System.out.println("Depois de ler o input stream fornecido pelo cliente");

      String data = inStream.readUTF();             // data
      System.out.println("Data="+ data);
      String tmp_wwwRootDir = inStream.readUTF();   // WWW root directory
      System.out.println("tmp_wwwRootDir="+ tmp_wwwRootDir);
      String tmp_baseDir = inStream.readUTF();      // base directory
      System.out.println("tmp_baseDir="+ tmp_baseDir);
      String tmp_host = inStream.readUTF();         // host
      System.out.println("tmp_host="+ tmp_host);
      int port = inStream.readInt();                // socket port
      int width = inStream.readInt();               // image width
      int height = inStream.readInt();              // image height
      int tmp_wakeupTime = inStream.readInt();      // wakeup time (minutes)

      if(init == false) {
        // set values for static parameters: wwwRootDir, imageWWWDir, wakeupTime
        wwwRootDir = tmp_wwwRootDir;
        System.out.println("wwwRootDir="+ wwwRootDir);
        baseDir = tmp_baseDir;
        wakeupTime = tmp_wakeupTime;
        host = tmp_host;

        // init filename generator: temp images
        fileGen = new TmpFilenameGenerator();
        System.out.println("Depois de passar pelo TmpFilenameGenerator wwwRootDir="+ wwwRootDir);
        // init file collector
        // specified interval for file collection
        //fileCollector = new TmpFileCollector(wwwRootDir + "/" + imageWWWDir, wakeupTime);
        TmpFileCollector fileCollector;
        fileCollector = new TmpFileCollector(wwwRootDir + "\\" + 
                                             baseDir + "\\" + 
                                             DEFAULT_IMAGE_DIRECTORY, 
                                             wakeupTime);

        init = true;
      }

      String imageName = fileGen.nextFilename();

      //
      // processing goes here...
      //

      // create new image
      int size = width * height;
      int[] pixels = new int[size];

      Frame frame = new Frame();
      frame.pack();
      Image image = frame.createImage(width, height);
      Graphics g = image.getGraphics(); 
      Dimension imageSize = new Dimension(width, height);

      // draw image ...
      StringBufferInputStream istream = new StringBufferInputStream(data);
      Share share = new Share();
      share.drawProject(g, imageSize, istream, null);
      //g.drawRect(20,20,60,60);

      // ... and convert it to gif file
      File output = new File(wwwRootDir + "\\" + baseDir + "\\" + 
                             DEFAULT_IMAGE_DIRECTORY + "\\" + imageName);
      System.out.println("Diretorio=" +  wwwRootDir + "\\" + baseDir + "\\" + DEFAULT_IMAGE_DIRECTORY + "\\" + imageName);
      System.out.println("OUTPUT="+ output.toString());
      FileOutputStream fos = new FileOutputStream(output);
      System.out.println("Depois de passar pelo construtor FileOutPutStream");
      Date now = new Date();
      System.out.println("Antes de chamar o construtor GifEncoder");
      GifEncoder encoder = new GifEncoder(image, fos);
      System.out.println("Depois de chamar GifEncoder e antes de chamar metodo GifEncoder");
      encoder.encode();
      System.out.println("Depois de chamar metodo GifEncoder");
      fos.close();

      // return image URL
      //String url = "http://" + host + "/" + baseDir + "/" + 
      //              DEFAULT_IMAGE_DIRECTORY + "/" + imageName;
      StringBuffer dest = new StringBuffer(baseDir.length());
      int i;
      for (i = 0;  i < baseDir.length(); i++) {
            if (baseDir.charAt(i) == '\\')
               dest.append('/');
            else
               dest.append(baseDir.charAt(i));  
      }
           
      String url = "http://" + host + "/" + dest.toString() + "/" +
                    DEFAULT_IMAGE_DIRECTORY + "/" + imageName;
      System.out.println("Url retornada para o cliente=" + url);
      outStream = new ObjectOutputStream(client.getOutputStream());
      outStream.writeInt(AgentError.OK);
      outStream.writeUTF(url);

      // close stream
      outStream.flush();
    }
    catch(IOException e) {
    	System.out.println("Ocorreu IOException no Agent Gif Encoder");
      e.printStackTrace();
    }
    catch(Exception e) {
      System.out.println("Ocorreu Exception que desconheco");
      e.printStackTrace();
    }
    finally {
      // alway close the connection
      try { client.close(); }
      catch(IOException ignored) {}
    }
  }

/*
   private String wwwDirectory(String root, String realPath)
   {
     if(realPath.startsWith(root)) {
       try {
         return realPath.substring(root.length(), realPath.length());
       }
       catch(StringIndexOutOfBoundsException e) {
         System.out.println("[AgentGifEncoder] error: <" + root + "> not in <" + realPath + ">");
         return null;
       }
     }
     else {
       System.out.println("[AgentGifEncoder] error: <" + root + "> not in <" + realPath + ">");
       return null;
     }
   }

   private String parentDirectory(String path)
   {
     try {
       return path.substring(0, path.lastIndexOf(File.separatorChar));
     }
     catch(StringIndexOutOfBoundsException e) {
       return null;
     }
   }
*/
}

