-- ----------------------------------------------------------------------------------------------------
-- CADASTRA.LUA
-- Responsavel pelo CADASTRAMENTO, EDICAO e REVISAO de projetos
-- ----------------------------------------------------------------------------------------------------
-- CARREGA BIBLIOTECAS
-- ----------------------------------------------------------------------------------------------------
assert(dofile( 'acessabd.lua' ))
assert(dofile( 'global.lua' ))
assert(dofile( 'constantes.lua' ))

if ESTADO_BD ~= 'aberto' then
   AbreBD()
end

-- ----------------------------------------------------------------------------------------------------
-- Inicializa o valor de flag_valor_custo_dif com 0. Esse flag eh utilizado para comparar os valores 
-- fornecidos em custo anual (tabela CUSTO) e os jah cadastrados em custo mensal (CUSTO_PREVISTO) - 
-- caso entradas para esse ultimo sejam registradas, verificando se eles sao diferentes
-- ----------------------------------------------------------------------------------------------------
flag_valor_custo_dif = 0 

data_hoje = date("%Y%m%d")
gdia      = strsub(data_hoje,7,8)
gmes      = strsub(data_hoje,5,6)
gano      = strsub(data_hoje,1,4)

-- ----------------------------------------------------------------------------------------
-- Esta funcao inicializa o valor do CGI referente ao TOTAL do custo previsto anual 
-- Eh utilizada tanto no CADASTRO quanto na EDICAO dos projetos
-- ----------------------------------------------------------------------------------------
function InicializaCustoPrevistoTotal()
   local ano
   if not( temConteudo( cgi.custo_total ) ) then
   	  ano = tonumber( _ANO_INI_PEGASO_ )
   	  cgi.custo_total = 0
      while ano <= tonumber( _ANO_FIM_PEGASO_ ) do 
         if temConteudo(cgi['custo_previsto_'..ano]) then
            cgi.custo_total = tonumber(cgi.custo_total) + tonumber(cgi['custo_previsto_'..ano])
         end   
         ano = ano + 1
      end     
   else
      cgi.custo_total = gsub(cgi.custo_total,",",".")
   end 
end   

-- ----------------------------------------------------------------------------------------
-- Esta funcao inicializa os valores dos CGIs referentes ao custo previsto anual 
-- Eh utilizada tanto no CADASTRO quanto na EDICAO dos projetos
-- ----------------------------------------------------------------------------------------
function InicializaCustoPrevistoAnual()
   local ano
   ano = tonumber( _ANO_INI_PEGASO_ )
   while ano <= tonumber( _ANO_FIM_PEGASO_ ) do 
   	  if temConteudo( cgi['custo_previsto_'..ano] ) then
         cgi['custo_previsto_'..ano] = gsub(cgi['custo_previsto_'..ano], ',', '.')
      else
         cgi['custo_previsto_'..ano] = 0
      end   
      ano = ano + 1
   end 
end      

-- ----------------------------------------------------------------------------------------
-- Esta funcao efetua as insercoes/atualizacoes na tabela CUSTO
-- Recebe os valores referentes ao codigo do projeto e ao FLAG flag_valor_custo_dif
-- Eh utilizada tanto no CADASTRO quanto na EDICAO dos projetos
-- ----------------------------------------------------------------------------------------
function AtualizaCustoPrevistoAnual(cod_proj, flag)
   local ano
   ano = tonumber( _ANO_INI_PEGASO_ )    
   while ano <= tonumber( _ANO_FIM_PEGASO_ ) do                   	
      if ( temConteudo(cgi['custo_previsto_'..ano]) ) and flag ~= 1 then               
      	 custo = ConsultaBD("custo","*","proj_cod = "..cod_proj.." and custo_ano = "..ano,nil)
      	 cgi['custo_previsto_'..ano] = gsub(cgi['custo_previsto_'..ano],",",".")   
         if custo and custo.custo_prev_ano then                              	
            AlteraBD("custo", "custo_prev_ano = "..cgi["custo_previsto_"..ano].." ",
                     "proj_cod = "..cod_proj.." AND custo_ano = "..ano.." ")         	                                                     	
         else	                                  
            EscreveBD("custo", "proj_cod,custo_ano,custo_prev_ano", " "..cod_proj..", "..ano..", "..
                      cgi["custo_previsto_"..ano].." ")                  
         end              
      end
      ano = ano + 1           
   end  
end       

-- ----------------------------------------------------------------------------------------
-- Esta funcao efetua as atualizacoes na tabela PROJ_OT. 
-- Recebe valores referentes a OT OPERACIONAL, CLASSIFICACAO, SUB-CLASSIFICACAO e suas OTs
-- Eh utilizada tanto no CADASTRO quanto na EDICAO dos projetos 
-- Armazena os valores referentes a CLASSIFICACAO e SUBCLASSIFICACAO mesmo que nao tenha
-- sido informado o valor da OT OPERACIONAL ou da OT ESPECIFICA
-- ----------------------------------------------------------------------------------------
function AtualizaOtOperacional(proj_cod, classificacao, ot_classificacao, subclass, ot_subclass, ot_oper)
   -- --------------------------------
   -- OT_CLASS (OT da classificao) 
   -- --------------------------------
   ot_class = ConsultaBD2("classificacao a , ot b","b.ot_cod as ot_class","a.clas_cod = "..classificacao.." and b.ot_chave = '"..ot_classificacao.."' ",nil )

   -- ----------------------------------------------------------------------------------------
   -- OBS: As chaves em PROJ_OT sao PROJ_COD e CLAS_COD
   -- CAMPOS: proj_cod, clas_cod, proj_ot_subcl_cod, ot_cod, proj_ot_ot_subclas, ot_oper_cod 
   -- ----------------------------------------------------------------------------------------
   cons = ConsultaBD2("proj_ot","*"," proj_cod = "..proj_cod.." AND clas_cod = "..classificacao.."", nil )
   
   -- --------------------------------------------------------
   -- MONTAGEM DOS CAMPOS PARA INCLUSO/ALTERAO EM proj_ot
   -- --------------------------------------------------------                     
   -- --------------------------------------------------------
   -- Classificacao e sua OT 
   -- --------------------------------------------------------            
   if temConteudo(classificacao) then
      altera        = " ot_cod = "..ot_class[1].ot_class
      condicao      = " proj_cod = "..proj_cod.." AND clas_cod = "..classificacao.." "
      escreve_campo = "proj_cod, clas_cod, ot_cod"
      escreve_valor = proj_cod..", "..classificacao..", "..ot_class[1].ot_class
   end   
   -- -------------------------------------------------------------------
   -- Se tem sub-classificacao e for diferente de branco, nulo ou vazio
   -- -------------------------------------------------------------------
   if temConteudo(subclass) then
      subclassificacao = ConsultaBD("ot a, classificacao b","b.clas_cod as subclas, a.ot_cod as ot_subclas","a.ot_chave = '"..ot_subclass.."' and a.clas_cod = b.clas_cod",nil )                  
      local ot_sub = subclassificacao.ot_chave
      -- --------------------------------------------
      -- Sub-Classificacao e sua OT 
      -- --------------------------------------------
      if (subclassificacao and temConteudo(subclassificacao.subclas) ) then
         altera        = altera..", proj_ot_subcl_cod = "..subclass..",  "..
                         " proj_ot_ot_subclas = "..subclassificacao.ot_subclas 
         escreve_campo = escreve_campo..", proj_ot_subcl_cod, proj_ot_ot_subclas "
         escreve_valor = escreve_valor..", "..subclass..", "..subclassificacao.ot_subclas             
      end                         
   else  -- if temConteudo(subclassificacao) 
      local ot_sub = ot_class[1].ot_chave
   end                                 
   -- --------------------------------------------
   -- OT Operacional
   -- --------------------------------------------            
   if ( temConteudo(ot_oper) ) then
      altera        = altera..", ot_oper_cod = "..ot_oper 
      escreve_campo = escreve_campo..", ot_oper_cod "
      escreve_valor = escreve_valor..", "..ot_oper
   end	                        
   -- --------------------------------------------
   -- Se jah consta na BASE DE DADOS
   -- --------------------------------------------                        
   if cons[1] and temConteudo(cons[1].proj_cod) then
      -- -------------------------------------------------------------------
      -- OT Operacional, Classificacao, subclassificacao e suas OTs
      -- -------------------------------------------------------------------
      if tonumber(classificacao) == tonumber(cons.clas_cod) then
         AlteraBD("proj_ot", altera, condicao)
      else
         ApagaBD("proj_ot", "proj_cod = "..proj_cod.." ")
         EscreveBD("proj_ot", escreve_campo, escreve_valor)
      end              
   else
      EscreveBD("proj_ot", escreve_campo, escreve_valor)
   end   
   
   return (ot_sub)
end -- fim function AtualizaOtOperacional()

-- Funo de Reviso do Projeto
function RevisaProj(codigo_projeto,versao)   
   datacad = DataAtual()
   -- ------------------------------------------------------------------------------------------------------- 
   -- Altera o valor do campo LAST_VERSAO do projeto que deu origem ao projeto revisado para 'n'
   -- e zera o contador de Follow-Up
   -- -------------------------------------------------------------------------------------------------------   
   AlteraBD("projeto", "last_versao = 'n', proj_cont_fup = 0" ," proj_cod = "..codigo_projeto.." ")

   --------------------------------
   --Consulta a tabela de pessoal
   --------------------------------
   cresp = ConsultaBD2("pessoal","*","pes_chave = '"..getuser().."'")

   --------------------------------
   --Consulta a tabela de projeto
   --------------------------------
   cproj = ConsultaBD2("projeto","*","proj_cod = "..codigo_projeto.."")

   ini_prev = inclui_data(cproj[1].proj_dt_ini_prev)
   fim_prev = inclui_data(cproj[1].proj_dt_fim_prev)

   campos = ""
   valores = ""

   campos = campos.."org_cod"
   valores = valores..cproj[1].org_cod
   
   campos = campos..",proj_nome"
   valores = valores..",'"..cproj[1].proj_nome.."'"

   campos = campos..",proj_versao"
   valores = valores..","..versao
   
   campos = campos..",stat_cod"
   valores = valores..","..1

   campos = campos..",proj_dt_incl"
   valores = valores..",'"..datacad.."'"

   campos = campos..",last_versao"
   valores = valores..",'s'"

   campos = campos..",proj_dt_ini_prev"
   valores = valores..",'"..ini_prev.."'"
 
   campos = campos..",proj_dt_fim_prev"
   valores = valores..",'"..fim_prev.."'"
   
   campos = campos..",proj_last_cod"
   valores = valores..","..codigo_projeto

   if cproj[1].proj_sigla then
      campos = campos..",proj_sigla"
      valores = valores..",'"..cproj[1].proj_sigla.."'"
   end
   
   if cproj[1].proj_prior then
      campos = campos..",proj_prior"
      valores = valores..","..cproj[1].proj_prior
   end   

   if cproj[1].proj_descr then
      campos = campos..",proj_descr"
      valores = valores..",'"..cproj[1].proj_descr.."'"
   end
   
   if cproj[1].proj_pae then
      campos = campos..",proj_pae"
      valores = valores..","..cproj[1].proj_pae
   end
   
   if cproj[1].unid_cod then
      campos = campos..",unid_cod"
      valores = valores..","..cproj[1].unid_cod
   end
     
   if cproj[1].proj_dt_ini_real and cproj[1].proj_dt_ini_real ~= '' then
      campos = campos..",proj_dt_ini_real"
      valores = valores..",'"..cproj[1].proj_dt_ini_real.."'"
   end
   
   if cproj[1].proj_dt_fim_real and cproj[1].proj_dt_fim_real ~= '' then
      campos = campos..",proj_dt_fim_real"
      valores = valores..",'"..cproj[1].proj_dt_fim_real.."'"
   end
   
   if cproj[1].proj_num_ote then
      campos = campos..",proj_num_ote"
      valores = valores..",'"..cproj[1].proj_num_ote.."'"
   end   
   
   if cproj[1].ger_cod then
      campos = campos..",ger_cod"
      valores = valores..","..cproj[1].ger_cod
   end
   
   if cproj[1].set_cod then
      campos = campos..",set_cod"
      valores = valores..","..cproj[1].set_cod
   end
   
   if cproj[1].proj_paeid then
      campos = campos..",proj_paeid"
      valores = valores..","..cproj[1].proj_paeid
   end
   
   if cproj[1].pes_cod then
      campos = campos..",pes_cod"
      valores = valores..","..cproj[1].pes_cod      
   end   
   
   -- ------------------------------------------------------------------------------------
   -- PROJ_RESP_CAD: o usuario responsavel pelo cadastramento do projeto ou pela revisao
   --                associa ao codigo do getuser
   -- ------------------------------------------------------------------------------------
   campos = campos..",proj_resp_cad"
   valores = valores..","..retornaPessoaCod(getuser())        
   
   if cproj[1].proj_coord then
      campos = campos..",proj_coord"
      valores = valores..",'"..cproj[1].proj_coord.."'"
   end
   
   -- -----------------------------------------------------------------------------------
   -- O colaborador deve ser alterado na revisao para o usuario responsavel pela revisao
   -- -----------------------------------------------------------------------------------
   -- PROJ_COLAB: o usuario que estah atualmente editando o projeto
   -- -----------------------------------------------------------------------------------
   if cproj[1].proj_colab then
      campos = campos..",proj_colab"
      valores = valores..",'"..getuser().."'"
   end    
       
   if cproj[1].proj_freq then
      campos = campos..",proj_freq"
      valores = valores..","..cproj[1].proj_freq
   end
   
   if cproj[1].proj_plano_rec then
      campos = campos..",proj_plano_rec"
      valores = valores..","..cproj[1].proj_plano_rec
   end
   
   if cproj[1].proj_class then
      campos = campos..",proj_class"
      valores = valores..","..cproj[1].proj_class
   end
   
   if cproj[1].proj_cod_usuario then
      campos = campos..",proj_cod_usuario"
      valores = valores..",'"..cproj[1].proj_cod_usuario.."'"
   end

   if cproj[1].proj_meta then
      campos = campos..",proj_meta"
      valores = valores..","..cproj[1].proj_meta
   end
   
   if cproj[1].proj_soma_ativ then
      campos = campos..",proj_soma_ativ"
      valores = valores..","..cproj[1].proj_soma_ativ
   end
   
   if cproj[1].proj_desempenho then
      campos = campos..",proj_desempenho"
      valores = valores..","..cproj[1].proj_desempenho
   end

   if cproj[1].proj_perc_fisico_planejado then
      campos = campos..",proj_perc_fisico_planejado"
      valores = valores..","..cproj[1].proj_perc_fisico_planejado
   end

   if cproj[1].proj_perc_fisico_realizado then
      campos = campos..",proj_perc_fisico_realizado"
      valores = valores..","..cproj[1].proj_perc_fisico_realizado
   end

   -- -----------------------------------------
   -- Cadastra o projeto com a nova versao 
   -- -----------------------------------------
   EscreveBD("projeto", campos, valores)  
   
   -- ---------------------------------------------------------
   -- Consulta o codigo do projeto incluido com nova versao 
   -- ---------------------------------------------------------
   dproj = ConsultaBD2("projeto","proj_cod","proj_sigla = '"..cproj[1].proj_sigla.."' and proj_versao = "..versao .."")

   -- --------------------------------------------------------------------------
   -- Cadastra os dados na tabela de "colaboradores" para o projeto revisado 
   -- O responsavel pelo cadastro antes da revisao (que eh tb o proj_colab)  
   --  inserido na tabela de colaboradores quando da revisao                
   -- --------------------------------------------------------------------------
   cod_proj = ConsultaBD2("projeto","proj_cod, proj_resp_cad","last_versao = 's' AND proj_nome = '"..cproj[1].proj_nome.."'", nil)
   if cproj[1].proj_resp_cad ~= cod_proj[1].proj_resp_cad then    
      EscreveBD("colaboradores", "proj_cod, colaborador", cod_proj[1].proj_cod..", "..cproj[1].proj_resp_cad.."") 
   end
  
   ccolaboradores = ConsultaBD2("colaboradores","*","proj_cod = "..codigo_projeto, nil)
   local z = 1
   while ccolaboradores[z] and ccolaboradores[z].colaborador do
      -- --------------------------------------------------------------------------------------------------
      -- Se for coordenador, colaborador atual ou o colaborador antigo no insere na tabela COLABORADORES.
      -- O colaborador antigo j foi inserido na tabela COLABORADORES acima.
      -- --------------------------------------------------------------------------------------------------
      if ((ccolaboradores[z].colaborador ~= cproj[1].proj_resp_cad) and (ccolaboradores[z].colaborador ~= cresp[1].pes_cod) and (ccolaboradores[z].colaborador ~= retornaPessoaCod(cproj[1].proj_coord)) ) then
         EscreveBD("colaboradores","proj_cod, colaborador"," "..dproj[1].proj_cod..", "..ccolaboradores[z].colaborador.." ")
      end   
      z = z + 1
   end
   
   -- -----------------------------------------------------------------
   -- Consulta os dados da tabela "proj_ot" para o projeto revisado 
   -- -----------------------------------------------------------------
   cprojot = ConsultaBD2("proj_ot","clas_cod,proj_ot_subcl_cod,ot_cod,proj_ot_ot_subclas,ot_oper_cod","proj_cod = "..codigo_projeto.."")

   -- --------------------------------------------------------------------
   -- Cadastra os dados na tabela de "proj_ot" para o projeto revisado 
   -- para OT ESPECIFICA, OT OPERACIONAL e projetos sem OT
   -- --------------------------------------------------------------------
   if not cprojot[1].proj_ot_subcl_cod then
      coluna = ""
      valor = ""
   else
      coluna = ",proj_ot_subcl_cod"
      valor = ","..cprojot[1].proj_ot_subcl_cod..""  
   end

   if not cprojot[1].proj_ot_ot_subclas then
      coluna = coluna..""
      valor = valor..""
   else
      coluna = coluna..",proj_ot_ot_subclas"
      valor = valor..","..cprojot[1].proj_ot_ot_subclas..""
   end

   if not cprojot[1].ot_oper_cod then
      coluna = coluna..""
      valor = valor..""
   else
      coluna = coluna..",ot_oper_cod"
      valor = valor..","..cprojot[1].ot_oper_cod..""
   end
   
   EscreveBD("proj_ot","proj_cod,clas_cod,ot_cod "..coluna.." ", " "..dproj[1].proj_cod..","..
             cprojot[1].clas_cod..", "..cprojot[1].ot_cod.." "..valor.." ")

   -- --------------------------------------------------------------------
   -- Consulta os dados da tabela "transpetro" para o projeto revisado 
   -- --------------------------------------------------------------------
   ctransp = ConsultaBD2("transpetro","*", "proj_cod = "..codigo_projeto.."")

   -- -----------------------------------------------------------------------
   -- Cadastra os dados na tabela de "transpetro" para o projeto revisado 
   -- -----------------------------------------------------------------------
   if ctransp[1].proj_cod then
      if not ctransp[1].trans_eng then
         coluna1 = ""
         valor1 = ""
      else
         coluna1 = ",trans_eng"
         valor1 = ",'"..ctransp[1].trans_eng.."'"
      end

      if not ctransp[1].trans_cbi then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_cbi"
         valor1 = valor1..",'"..ctransp[1].trans_cbi.."'"
      end

      if not ctransp[1].trans_cac then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_cac"
         valor1 = valor1..",'"..ctransp[1].trans_cac.."'"
      end

      if not ctransp[1].trans_cad then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_cad"
         valor1 = valor1..",'"..ctransp[1].trans_cad.."'"
      end

      if not ctransp[1].trans_ui then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_ui"
         valor1 = valor1..",'"..ctransp[1].trans_ui.."'"
      end

      if not ctransp[1].trans_prog then
         coluna1 = coluna1..""
         valor1 = valor1""
      else
         coluna1 = coluna1..",trans_prog"
         valor1 = valor1..",'"..ctransp[1].trans_prog.."'"
      end

      if not ctransp[1].trans_proj then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_proj"
         valor1 = valor1..",'"..ctransp[1].trans_proj.."'"
      end

      if not ctransp[1].trans_tipo then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_tipo"
         valor1 = valor1..",'"..ctransp[1].trans_tipo.."'"
      end
      EscreveBD("transpetro","proj_cod "..coluna1.." "," "..dproj[1].proj_cod.." "..valor1.." ")
   end   
  
   -- -------------------------------------------------------------
   -- Consulta os dados da tabela "log" para o projeto revisado 
   -- -------------------------------------------------------------
   clog = ConsultaBD2("log","pes_cod,log_justifica", "proj_cod = "..codigo_projeto.."")
  
   -- ----------------------------------------------------------------
   -- Cadastra os dados na tabela de "log" para o projeto revisado 
   -- ----------------------------------------------------------------
   if clog[1] and clog[1].pes_cod then
      EscreveBD("log","proj_cod,pes_cod,log_data,log_justifica", " "..dproj[1].proj_cod..","..
                 clog[1].pes_cod..", '"..datacad.."','"..clog[1].log_justifica.."' ")
   end
  
   -- ---------------------------------------------------------------
   -- Consulta os dados da tabela "dutos" para o projeto revisado 
   -- Coloca o resultado na TABELA cdutos                         
   -- ---------------------------------------------------------------
   cdutos = ConsultaBD2("dutos","duto_nome,duto_diametro,duto_espessura,duto_extensao,duto_materiais,duto_origem,duto_destino,duto_produto,duto_outras","proj_cod = "..codigo_projeto.." ")

   -- ------------------------------------------------------------------
   -- Cadastra os dados na tabela de "dutos" para o projeto revisado 
   -- ------------------------------------------------------------------   
   coluna2 = ""
   valor2 = ""
  
   if cdutos[1].duto_nome then
      coluna2 = coluna2..",duto_nome"
      valor2 = valor2..",'"..cdutos[1].duto_nome.."'"
   end
   
   if cdutos[1].duto_diametro then
      coluna2 = coluna2..",duto_diametro"
      valor2 = valor2..","..cdutos[1].duto_diametro..""
   end
   
   if cdutos[1].duto_espessura then
      coluna2 = coluna2..",duto_espessura"
      valor2 = valor2..","..cdutos[1].duto_espessura..""
   end
 
   if cdutos[1].duto_extensao then
      coluna2 = coluna2..",duto_extensao"
      valor2 = valor2..","..cdutos[1].duto_extensao..""
   end
 
   if cdutos[1].duto_materiais then
      coluna2 = coluna2..",duto_materiais"
      valor2 = valor2..",'"..cdutos[1].duto_materiais.."'"
   end
   
   if cdutos[1].duto_origem then
      coluna2 = coluna2..",duto_origem"
      valor2 = valor2..",'"..cdutos[1].duto_origem.."'"
   end
 
   if cdutos[1].duto_destino then
      coluna2 = coluna2..",duto_destino"
      valor2 = valor2..",'"..cdutos[1].duto_destino.."'"
   end
   
   if cdutos[1].duto_produto then
      coluna2 = coluna2..",duto_produto"
      valor2 = valor2..",'"..cdutos[1].duto_produto.."'"
   end
   
   if cdutos[1].duto_outras then
      coluna2 = coluna2..",duto_outras"
      valor2 = valor2..",'"..cdutos[1].duto_outras.."'"
   end
 
   if cdutos[1] and cdutos[1] ~= '' then
      EscreveBD("dutos","proj_cod "..coluna2.." ", " "..dproj[1].proj_cod.." "..valor2.." ")
   end
    
   -- ---------------------------------------------------------------------------------
   -- Consulta os dados da tabela de custo referente ao projeto passado como parametro              
   ------------------------------------------------------------------------------------
   ccusto = ConsultaBD2("custo","*","proj_cod = "..codigo_projeto.."","custo_ano")
    
   -- -----------------------------------------------------------------
   -- Cadastra os dados de custo anual previsto do projeto revisado 
   -- -----------------------------------------------------------------
   i = 1
   while ccusto[i] do
      EscreveBD("custo","proj_cod,custo_ano,custo_prev_ano",""..dproj[1].proj_cod..","..tonumber(ccusto[i].custo_ano)..","..tonumber(ccusto[i].custo_prev_ano).."")
      i = i + 1
   end
   
   -- -------------------------------------------------------------------------------------------
   -- Consulta os dados da tabela de custo_previsto referente ao projeto passado como parametro                      
   -- -------------------------------------------------------------------------------------------
   cmcusto = ConsultaBD2("custo_previsto","dt_prevista,valor_real","proj_cod = "..codigo_projeto.."","dt_prevista")  
  
   -- ------------------------------------------------------------------
   -- Cadastra os dados de custo mensal previsto do projeto revisado 
   -- ------------------------------------------------------------------
   i = 1
   while cmcusto[i] do
      if cmcusto[i].dt_prevista then 	
         -- -------------------------------------------------------------------------------------------------------           	
         -- Remove as barras colocando no formato AAAAMMDD (inclui_data) e substitui brancos por "0" (formataData)
         -- -------------------------------------------------------------------------------------------------------           	
         cmdata = inclui_data(cmcusto[i].dt_prevista)
         EscreveBD("custo_previsto","proj_cod,dt_prevista,valor_real",""..dproj[1].proj_cod..",'"..cmdata.."',"..cmcusto[i].valor_real.."")           
      end   
      i = i + 1
   end

   -- -------------------------------------------------------------------------------------------
   -- Consulta os dados da tabela de custo_realizado referente ao projeto passado como parametro       
   -- -------------------------------------------------------------------------------------------
   crcusto = ConsultaBD2("custo_realizado", "*", "proj_cod = "..codigo_projeto.."", "dt_realizacao")
  
   -- ------------------------------------------------------------
   -- Cadastra os dados de custo realizado do projeto revisado 
   -- ------------------------------------------------------------
   i = 1
   while crcusto[i] do
      if crcusto[i].dt_realizacao then
         -- -------------------------------------------------------------------------------------------------------
         -- Remove as barras colocando no formato AAAAMMDD (inclui_data) e substitui brancos por "0" (formataData)                       
         -- -------------------------------------------------------------------------------------------------------
         crdata1 = inclui_data(crcusto[i].dt_realizacao)
         crdata2 = gano..gmes..gdia
         EscreveBD ("custo_realizado", "proj_cod, dt_realizacao, dt_inclusao, valor_real, seq_real", " "..dproj[1].proj_cod..", '"..crdata1.."', '"..crdata2.."', "..crcusto[i].valor_real..", "..crcusto[i].seq_real.." ")
      end
      i = i + 1
   end
   
   -- --------------------------------------------------
   -- Consulta as tabelas de atividade e dependencia 
   -- --------------------------------------------------
   cativ = ConsultaBD2("atividade a, dependencia b, perc_realizado c","a.ativ_cod, c.dt_atual, a.ativ_nome, "..
                       "a.ativ_dt_ini_prev, a.ativ_dt_fim_prev, a.ativ_dt_ini_real, a.ativ_dt_fim_real, "..
                       "a.ativ_peso_rel_proj,a.ativ_perc_real, a.ativ_alerta, b.dep_area_cod, b.dep_descr, "..
                       "c.perc_real"," "..
                       "a.proj_cod = "..codigo_projeto.." and a.ativ_cod *= b.dep_ativ_cod and "..
                       "a.ativ_cod *= c.ativ_cod and "..
                       "c.dt_atual = (SELECT MAX(dt_atual) FROM perc_realizado WHERE perc_realizado.ativ_cod=a.ativ_cod)"," "..
                       "a.ativ_cod")

   -- ------------------------------------------------------------------
   -- Cadastra as atividades, dependencias e o per_realizado, se tiver 
   -- ------------------------------------------------------------------
   i = 1
   while cativ[i] and cativ[i].ativ_cod do	  
      coluna3 = ""
      valor3 = ""
      if cativ[i].ativ_dt_ini_real and cativ[i].ativ_dt_ini_real ~= '' then
         coluna3 = coluna3..", ativ_dt_ini_real"
         valor3 = valor3..", '"..cativ[i].ativ_dt_ini_real.."' "
      end

      if cativ[i].ativ_dt_fim_real and cativ[i].ativ_dt_fim_real ~= '' then
         coluna3 = coluna3..", ativ_dt_fim_real"
         valor3 = valor3..", '"..cativ[i].ativ_dt_fim_real.."' "
      end
     
      if cativ[i].ativ_perc_real and cativ[i].ativ_perc_real ~= '' then
         coluna3 = coluna3..", ativ_perc_real"
         valor3 = valor3..", "..cativ[i].ativ_perc_real.." "
      end
        
      EscreveBD("atividade","proj_cod, ativ_nome, ativ_dt_ini_prev, ativ_dt_fim_prev "..coluna3..", "..
                " ativ_peso_rel_proj, ativ_alerta ", " "..
                " "..dproj[1].proj_cod..", '"..cativ[i].ativ_nome.."', '"..cativ[i].ativ_dt_ini_prev.."', "..
                " '"..cativ[i].ativ_dt_fim_prev.."' "..valor3..", "..
                " "..cativ[i].ativ_peso_rel_proj..", "..cativ[i].ativ_alerta.." ")
      
      -- -----------------------------------------------------------
      -- Consulta o codigo da atividade para incluir a dependencia 
      -- e o perc_realizado correspondente 
      -- -----------------------------------------------------------
      dativ = ConsultaBD2("atividade","ativ_cod","proj_cod = "..dproj[1].proj_cod.." and ativ_nome = '"..cativ[i].ativ_nome.."'")
      
      -- -------------------------------------
      -- Grava as dependencias que existirem
      -- -------------------------------------
      if cativ[i].dep_area_cod then
         EscreveBD ("dependencia", "dep_proj_cod, dep_ativ_cod, dep_area_cod, dep_descr", " "..dproj[1].proj_cod..", "..
                    " "..dativ[1].ativ_cod..", "..cativ[i].dep_area_cod..", '"..cativ[i].dep_descr.."' ")
      end
      
      -- ---------------------------------------------
      -- Grava os dados existentes na perc_realizado
      -- ---------------------------------------------
      if cativ[i].perc_real then
      	 --print (cativ[i].dt_atual) exit()
      	 EscreveBD ("perc_realizado", "proj_cod, ativ_cod, dt_atual, perc_real", " "..dproj[1].proj_cod..", "..
      	            " "..dativ[1].ativ_cod..", '"..date('%c').."', "..cativ[i].perc_real.." ")
      end
      	            
      -- -------------------------------------------------------------------------------------------------
      -- Consulta a tabela de observacoes para verificar se h ou nao observacao para a atividade gravada      
      ----------------------------------------------------------------------------------------------------
      cobs = ConsultaBD2 ("observacoes", "obs_titulo, obs_descr, obs_versao, obs_status, obs_resp, obs_sigla, obs_tipo", "obs_proj_cod = "..codigo_projeto.." and obs_ativ_cod = "..cativ[i].ativ_cod.." ")
      
      -- ---------------------------------------------------------------
      -- Cadastra a observaco relativa a atividade que foi revisada 
      -- ---------------------------------------------------------------
      j = 1
      while cobs[j] and cobs[j].obs_titulo do
         data = gano..gmes..gdia
         EscreveBD ("observacoes", "obs_titulo, obs_proj_cod, obs_ativ_cod, obs_descr, obs_data, obs_versao, "..
                    "obs_status, obs_resp, obs_sigla, obs_tipo", " '"..cobs[j].obs_titulo.."', "..
                    ""..dproj[1].proj_cod..", "..dativ[1].ativ_cod..", '"..cobs[j].obs_descr.."', "..
                    "'"..data.."', "..cobs[j].obs_versao..", "..cobs[j].obs_status..", "..
                    "'"..cobs[j].obs_resp.."', "..cobs[j].obs_sigla..", "..cobs[j].obs_tipo.." ")
         j = j + 1
      end
      i = i + 1
   end
   
   -- ------------------------------------
   -- Consulta a tabela de observacoes 
   -- ------------------------------------
   dobs = ConsultaBD2 ("observacoes", "obs_titulo, obs_descr, obs_data, obs_versao, obs_status, obs_resp, "..
                       "obs_sigla, obs_tipo", "obs_proj_cod = "..codigo_projeto.." and obs_ativ_cod is NULL") 
   
   -- -------------------------------------
   -- Cadastra as observacoes, se tiver 
   -- -------------------------------------
   i = 1
   while dobs[i] and dobs[i].obs_titulo do
      EscreveBD ("observacoes","obs_titulo, obs_proj_cod, obs_descr, obs_data, "..
                 " obs_versao, obs_status, obs_resp, obs_sigla, obs_tipo", " '"..dobs[i].obs_titulo.."', "..
                 " "..dproj[1].proj_cod..",'"..dobs[i].obs_descr.."', '"..dobs[i].obs_data.."', "..
                 " "..dobs[i].obs_versao..","..dobs[i].obs_status..", '"..dobs[i].obs_resp.."', "..
                 " "..dobs[i].obs_sigla..", "..dobs[i].obs_tipo.." ")
      i = i + 1
   end
      
   -- Grava follow_up do projeto que foi revisado
   atualizaFollowUp(codigo_projeto, 1, 'O projeto foi revisado.')  
          
   -- Grava follow_up do novo projeto resultante da revisao           
   atualizaFollowUp(dproj[1].proj_cod, 1, 'Reviso do projeto (nova verso).')  
                  
   return dproj[1].proj_cod
end -- function RevisaProj

if cgi.revisao then
   novo_codigo_projeto = RevisaProj(cgi.cod, cgi.versao)
   if ESTADO_BD == 'aberto' then
      FechaBD()
   end         
   cgilua.redirect('../cadastra.html?'..cgilua.encodetable({revisao = cgi.revisao, msgsuc = cgi.msgsuc, email = cgi.email, proj_cod = novo_codigo_projeto,tipo_servico = 'edita',tipo_edi='projeto'})) 

-- ----------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------
-- CADASTRA.LUA  (CADASTRO/EDICAO)
-- ----------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------

elseif cgi.botao == 'Grava' or cgi.botao == 'Detalhar_Custo' or cgi.botao == 'Atividades' or 
       cgi.botao == 'Grava Novo Projeto' or cgi.botao == 'Obs' or cgi.botao == 'Dutos' or
       cgi.botao == 'Formaliza' then

   if not( temConteudo(cgi.setor) ) then
      cgi.setor = 'NULL'
   end
  
   -- ---------------------------------------------------------------------------------------
   -- Inicializa o valor do CGI referente ao TOTAL do Custo Previsto Anual (cgi.custo_total) 
   -- --------------------------------------------------------------------------------------- 
   InicializaCustoPrevistoTotal()
  
   if temConteudo(cgi.descricao) then
      cgi.descricao = forca_plic(cgi.descricao) 
   else
      cgi.descricao = 'NULL'
   end  

   -- ----------------------------------
   -- CARREGA A BIBLIOTECA VALIDAFORM
   -- ----------------------------------
   assert(dofile("../validaform/validaform.lua"))
   assert(dofile("form_cadastra.lua"))
     
   cgi.msg = form_cadastra:critica()   
   
   cgi.plano_emerg = 0
   if tonumber(cgi.plano_emerg) == 1 and tonumber(cgi.plano_emerg_duto) == 1 then
      cgi.msg = 'O formulrio foi preenchido incorretamente. Siga as instrues junto aos campos que devem ser revistos.'
      cgi.msg_plano_emerg_erro = 'Um projeto no pode pertencer a ambos planos emergenciais.'
   end   
   if cgi.coordenador and cgi.coordenador ~= '' then
      confere_coord = "SELECT pes_chave FROM pessoal WHERE pes_chave = '"..cgi.coordenador.."'"
      DBExec(confere_coord)
      resultado_coord = DBRow()
      if not resultado_coord or not resultado_coord.pes_chave then
         cgi.msg_coordenador = 'O usurio no consta nos cadastros.'
         if not cgi.msg then
            cgi.msg = 'O formulrio foi preenchido incorretamente. Siga as instrues junto aos campos que devem ser revistos.'         
         end
      end
   end  
   
   -- ----------------------------------------------------------------------------
   -- Verifica se o valor de custo anual  diferente do valor do custo mensal. 
   -- ---------------------------------------------------------------------------- 
   if cgi.tipo_servico == 'edita' then     
      consulta_custo_previsto = "SELECT proj_cod FROM custo_previsto WHERE proj_cod = " .. cgi.proj_cod
      DBExec(consulta_custo_previsto)
      resultado_custo_previsto = DBRow()      
      if ((resultado_custo_previsto) and (resultado_custo_previsto.proj_cod)) then
         i = tonumber( _ANO_INI_PEGASO_ )
         while ( i <= tonumber( _ANO_FIM_PEGASO_ ) ) do
            local consulta = "SELECT SUM(valor_real) as valor_real FROM custo_previsto WHERE year(dt_prevista) = "..i..
                             " AND proj_cod = "..cgi.proj_cod
            DBExec(consulta)
            local resultado = DBRow()
            
            -- -----------------------------------------------------------------------------------------
            -- Verifica se o somatrio de custo mensal para um dado ano em um projeto no  NULL (nil)
            -- -----------------------------------------------------------------------------------------             
            if resultado and resultado.valor_real == nil then            	
            	som_custo_mensal = 0
            else               
               som_custo_mensal = resultado.valor_real
            end 	
            
            cgi['custo_previsto_'..i] = gsub(cgi['custo_previsto_'..i],',','.')  
            if (tonumber(cgi['custo_previsto_'..i]) ~= tonumber(som_custo_mensal)) then            	
            	flag_valor_custo_dif = 1
            end
            i = i + 1
         end      
         if cgi.botao ~= "Detalhar_Custo" then            	
            if (flag_valor_custo_dif == 1) then
               cgi.msg_custo_anual_dif = 'Os valores de custo anual esto diferentes dos valores detalhados em custo mensal.<BR>V em "<i>detalhar custos</i>" para corrigir esses valores.'
               if not (cgi.msg) then
               	cgi.msg = 'O formulrio foi preenchido incorretamente. Siga as instrues junto aos campos que devem ser revistos.'
               end  
            end         
         end      
      end
      
   end
   
   -- ----------------------------------------------------------------------  
   -- HOUVE ERRO NO PREENCHIMENTO DO FORMULRIO DE CADASTRO
   -- ----------------------------------------------------------------------  
   if cgi.msg then 
      if ESTADO_BD == 'aberto' then
         FechaBD()
      end           
      cgilua.redirect('../cadastra.html?'..cgilua.encodecgi())   -- VOLTA PARA A PGINA html--

   elseif temConteudo(cgi.ot_oper) and temConteudo(cgi.ot_especifica) then
      cgi.msg ='Escolha OT Operacional ou OT Especfica. Essa escolha  exclusiva!'
      if ESTADO_BD == 'aberto' then
         FechaBD()
      end 
      cgilua.redirect('../cadastra.html?'..cgilua.encodecgi())    -- VOLTA PARA A PGINA html--
         
   else -- NAO HOUVE ERRO NA VALIDACAO DO FORMULARIO

      cgi.dt_ini_prevista = formataData(cgi.dt_ini_prevista)
      cgi.dt_fim_prevista = formataData(cgi.dt_fim_prevista)
      dataini = strsub(cgi.dt_ini_prevista,7,10)..strsub(cgi.dt_ini_prevista,4,5)..strsub(cgi.dt_ini_prevista,1,2)
      datafim = strsub(cgi.dt_fim_prevista,7,10)..strsub(cgi.dt_fim_prevista,4,5)..strsub(cgi.dt_fim_prevista,1,2)
      anoIni  = tonumber( strsub(dataini,1,4) ) 
      anoFim  = tonumber( strsub(datafim,1,4) )
      datacad = DataAtual()
 
      if temConteudo(cgi.proj_cod) and not strfind(cgi.proj_cod,",") and cgi.botao ~= 'Grava Novo Projeto' then
         
         -- -----------------------
         -- inicio UPDATE (EDIO)
         -- -----------------------
         
         orgao = ConsultaBD('orgao','*'," org_cod = '"..cgi.orgao.."' ",nil)      

         if cgi.unidade and cgi.unidade ~= '' and cgi.unidade ~= ' ' then
            unidade = ConsultaBD('unidade','*'," unid_cod = '"..cgi.unidade.."' ",nil)      
            resp = unidade.pes_cod_resp
            cgi.resp = resp
            cgi.unid_cod = cgi.unidade
         else
            resp = orgao.pes_cod_resp
            cgi.resp = resp
         end

         if unidade and unidade ~= '' and unidade ~= ' ' then
            gerencia = "NULL" 

            if cgi.gerencia == "NULL" or cgi.gerencia == "" then
               gerencia = "NULL"
            elseif gerencia ~= "NULL" or cgi.gerencia then
               gerencia = tonumber(cgi.gerencia)
            end                       
            setor = "NULL"
            if cgi.setor == "NULL" or cgi.setor == "" then
               setor = "NULL"
            elseif setor ~= "NULL" or cgi.setor then
               setor = tonumber(cgi.setor)
            end
       
            -- ------------------------------------------------------------------
            -- Inicializa os valores dos CGIs referentes ao Custo Previsto Anual 
            -- ------------------------------------------------------------------            
            InicializaCustoPrevistoAnual()                                               

            -- ---------------------------------------------------------------------------------------
            -- Inicializa o valor do CGI referente ao TOTAL do Custo Previsto Anual (cgi.custo_total)  
            -- ---------------------------------------------------------------------------------------
            InicializaCustoPrevistoTotal()
            
            campos =       " org_cod = "..cgi.orgao..", "..
                           " proj_nome = '"..cgi.projeto.."', "..
                           " proj_prior = "..cgi.prioridade..", "..
                           " proj_coord = '"..cgi.coordenador.."', "..
                           " proj_colab = '"..getuser().."', "..    
                           " proj_freq = "..cgi.freq..", "..
                           " proj_dt_ini_prev = '"..dataini.."', "..
                           " proj_dt_fim_prev = '"..datafim.."', "..
                           " unid_cod = "..cgi.unidade..", "..
                           " ger_cod = "..gerencia..", "..
                           " set_cod = "..setor..", "..                                                      
                           " pes_cod = "..resp..", "..                                                  
                           " stat_cod = 1, "..
                           " proj_descr = '"..forca_plic(cgi.descricao).."', "..
                           " proj_pae = "..cgi.plano_emerg..", "..
                           " proj_paeid = "..cgi.plano_emerg_duto..", "..
                           " proj_meta = "..cgi.metas..", "..
                           " proj_num_ote = '"..cgi.ot_especifica.."'"

            if temConteudo(cgi.plano_rec) then
               campos = campos ..", proj_plano_rec = "..cgi.plano_rec
            end
            if temConteudo(cgi.class_acomp) then
               campos = campos ..", proj_class = "..cgi.class_acomp
            end
            if temConteudo(cgi.cod_usuario) then
               campos = campos ..", proj_cod_usuario = '"..cgi.cod_usuario.."'"
            end                                                     
            if temConteudo(cgi.ot_especifica) then               
               -- ---------------------------------------------------------------------------------------------------
               -- A existencia da OT ESPECIFICA implica na nao existencia da OT OPERACIONAL, logo, eh apagada na ta-
               -- bela PROJ_OT (informacoes referentes a OT OPERACIONAL) qq entrada referente ao PROJETO em questao
               -- Esse registro vai ser refeito pela execucao mais adiante da funcao AtualizaOtOperacional           
               -- ---------------------------------------------------------------------------------------------------
               cons = ConsultaBD("proj_ot","*"," proj_cod = "..cgi.proj_cod.." ", nil )
               if cons and temConteudo(cons.proj_cod) then
                  ApagaBD("proj_ot", " proj_cod = "..cons.proj_cod.." ")
                  cgi.ot_oper = ''
               end
            end
            
            AlteraBD("projeto", campos, " proj_cod = "..cgi.proj_cod.." ")  
                                                                          
         else
            unidadenula  = "NULL"
            gerencianula = "NULL"
            setornulo    = "NULL"

            -- ---------------------------------------------------------------------------------------
            -- Inicializa o valor do CGI referente ao TOTAL do Custo Previsto Anual (cgi.custo_total)  
            -- ---------------------------------------------------------------------------------------
            InicializaCustoPrevistoTotal()
            
            campos = " org_cod = "..cgi.orgao..", "..
                     " unid_cod = "..unidadenula.." ,"..
                     " ger_cod = "..gerencianula.." ,"..
                     " set_cod = "..setornulo.." ,"..
                     " proj_nome = '"..cgi.projeto.."', "..
                     " proj_prior = "..cgi.prioridade..", "..
                     " proj_coord = '"..cgi.coordenador.."', "..
                     " proj_colab = '"..getuser().."', "..
                     " proj_freq = "..cgi.freq..", "..
                     " proj_dt_ini_prev = '"..dataini.."', "..
                     " proj_dt_fim_prev = '"..datafim.."', "..
                     " pes_cod = "..resp..", "..                     
                     " stat_cod = 1, "..
                     " proj_descr = '".. cgi.descricao .."', "..
                     " proj_pae = "..cgi.plano_emerg..", "..
                     " proj_paeid = "..cgi.plano_emerg_duto..", "..
                     " proj_meta = "..cgi.metas..", "..
                     " proj_num_ote = '"..cgi.ot_especifica.."'"                                             

            if temConteudo(cgi.plano_rec) then
               campos = campos ..", proj_plano_rec = "..cgi.plano_rec
            end
            if temConteudo(cgi.class_acomp) then
               campos = campos ..", proj_class = "..cgi.class_acomp
            end
            if temConteudo(cgi.cod_usuario) then
               campos = campos ..", proj_cod_usuario = '"..cgi.cod_usuario.."'"
            end
            if temConteudo(cgi.ot_especifica) then               
               -- ---------------------------------------------------------------------------------------------------
               -- A existencia da OT ESPECIFICA implica na nao existencia da OT OPERACIONAL, logo, eh apagada na ta-
               -- bela PROJ_OT (informacoes referentes a OT OPERACIONAL) qq entrada referente ao PROJETO em questao
               -- Esse registro vai ser refeito pela execucao mais adiante da funcao AtualizaOtOperacional           
               -- ---------------------------------------------------------------------------------------------------
               cons = ConsultaBD("proj_ot","*"," proj_cod = "..cgi.proj_cod.." ", nil )
               if cons and temConteudo(cons.proj_cod) then
                  ApagaBD("proj_ot", " proj_cod = "..cons.proj_cod.." ")
                  cgi.ot_oper = ''
               end
            end

            AlteraBD("projeto", campos, " proj_cod = "..cgi.proj_cod.." ")
                                           
         end
         
         -- -------------------------------------------------------------------------------------
         -- OT OPERACIONAL
         -- Atualiza os dados referentes a CLASSIFICACAO, SUB e OT OPERACIONAL na tabela PROJ_OT
         -- -------------------------------------------------------------------------------------
         cgi.ot_sub = AtualizaOtOperacional(cgi.proj_cod, cgi.classificacao, cgi.ot_class, cgi.subclassificacao, cgi.ot_subclass, cgi.ot_oper)            
          
         -- ----------------------------------------------------------------------------------------
         -- Esta funcao efetua as insercoes/atualizacoes na tabela CUSTO
         -- Recebe os valores referentes ao codigo do projeto e ao FLAG flag_valor_custo_dif
         -- ----------------------------------------------------------------------------------------
         AtualizaCustoPrevistoAnual(cgi.proj_cod, flag_valor_custo_dif)
 
         -- -----------------------------------------------------------------------------
         -- CUSTO PREVISTO COM DETALHAMENTO MENSAL 
         -- Remove os valores inconsistentes com as datas de inicio e termino previstos
         -- OBS: dataini e datafim estao no formato AAAAMMDD                  
         -- -----------------------------------------------------------------------------         
         if dataini and datafim then         	
            sql_delete = " DELETE FROM custo_previsto WHERE proj_cod = "..cgi.proj_cod.." AND ( dt_prevista < '"..
                           dataini.."' OR dt_prevista > '"..datafim.."' ) "                                       
            DBExec(sql_delete)
         end   
         
         -- -----------------------------------------------------------------------------
         -- CUSTO PREVISTO ANUAL
         -- Remove os valores inconsistentes com as datas de inicio e termino previstos
         -- OBS: anoIni e anoFim estao no formato AAAA                  
         -- -----------------------------------------------------------------------------                 
         if dataini and datafim then        
            AlteraBD("custo", "custo_prev_ano = 0 ",
                     "proj_cod = "..cgi.proj_cod.." AND (custo_ano < "..anoIni.." OR custo_ano > "..anoFim..") ")                                      
         end                                                               
        
      else   --  if temConteudo(cgi.proj_cod) and not strfind(cgi.proj_cod,",") and cgi.botao ~= 'Grava Novo Projeto'
               
         -- -----------------------------
         -- NOVO PROJETO (CADASTRAMENTO)
         -- -----------------------------
          
         resp_cad = retornaPessoaCod(getuser())
         orgao = ConsultaBD('orgao','*'," org_cod = '"..cgi.orgao.."' ",nil)      
   
         if temConteudo(cgi.unidade) then
            unidade = ConsultaBD('unidade','*'," unid_cod = '"..cgi.unidade.."' ",nil)      
            resp = unidade.pes_cod_resp
            cgi.resp = resp
            cgi.unid_cod = cgi.unidade
         else
            resp = orgao.pes_cod_resp
            cgi.resp = resp
            cgi.unidade = nil
         end   
         -------------------------------------
         -- Verifica se o projeto jah existe 
         -------------------------------------
         if cgi.versao then
            versao = cgi.versao
         else
            versao = 0
         end
         ------------------------------------------------------------------------------
         -- toda inclusao de novo projeto considera o mesmo como sendo a ultima versao
         ------------------------------------------------------------------------------ 
         last_versao='s'
         
         ----------------------------------------------------------------
         -- Verifica a maior sigla e acrescenta + 1 para o projeto novo
         ----------------------------------------------------------------
         sigla = ConsultaBD2("projeto","proj_sigla",nil,"proj_sigla")
         if sigla[1].proj_sigla and sigla[1].proj_sigla ~= '' then
            i = 1
            num_aux = 0
            while sigla[i] do
               num = tonumber(gsub(sigla[i].proj_sigla,'P',''))
               if num and num > num_aux then
                  num_aux = num
               end
               i = i + 1
            end 
            proj_sigla = 'P'..tonumber(num_aux) + 1
         else
            proj_sigla = 'P1'
         end
      
         proj_unico = ConsultaBD("projeto","*"," org_cod = "..tonumber(cgi.orgao).." AND  proj_nome = '"..cgi.projeto.."' ","proj_versao")
         if temConteudo(proj_unico.proj_nome) then
            cgi.msg_proj = '<TR><TD>&nbsp;</TD><TD>&nbsp;</TD><TD><FONT CLASS=msgErro>Nome de Projeto j Cadastrado.</FONT></TD></TR>'
            if ESTADO_BD == 'aberto' then
               FechaBD()
            end 
            cgilua.redirect('../cadastra.html?'..cgilua.encodecgi())    -- VOLTA PARA A PGINA html --
          
         elseif temConteudo(unidade) then
            if temConteudo(cgi.gerencia) then
               gerencia = cgi.gerencia
            else
               gerencia = 'null'
            end
                            
            if temConteudo(cgi.setor) then
               setor = cgi.setor
            else
               setor = 'null'
            end
            -- ---------------------------------------------------------------------------------------
            -- Inicializa o valor do CGI referente ao TOTAL do Custo Previsto Anual (cgi.custo_total) 
            -- ---------------------------------------------------------------------------------------                 
            InicializaCustoPrevistoTotal()

            if temConteudo(cgi.gerencia) then  
               -- ------------------------------------------------------------------------------------------
               -- APRESENTA CAMPO UNIDADE, GER_COD e SET_COD                
               -- Verifica se existem campos nulos antes de inserir no BD para evitar erros no ACESSABD.LUA                         
               -- ------------------------------------------------------------------------------------------
               campos  = "org_cod, proj_nome, proj_prior, proj_dt_ini_prev, proj_dt_fim_prev, proj_dt_incl, unid_cod, "..
                         "ger_cod, pes_cod, proj_versao, stat_cod, proj_pae, proj_paeid,last_versao, proj_sigla, "..
                         "proj_resp_cad, proj_coord, proj_colab, proj_freq, proj_meta, proj_num_ote"
               valores = cgi.orgao..", '"..cgi.projeto.."', "..cgi.prioridade..", '"..dataini.."','"..datafim.."','"..
                         datacad.."',"..cgi.unidade..","..cgi.gerencia..","..resp..", "..versao..", 1, "..
                         cgi.plano_emerg..","..cgi.plano_emerg_duto..",'"..last_versao.."', '"..proj_sigla.."',"..
                         resp_cad..", '"..cgi.coordenador.."', '"..getuser().."', "..cgi.freq..", "..cgi.metas..",'"..
                         cgi.ot_especifica.."' "
               if cgi.descricao ~= 'NULL' then
                  campos  = campos .. ", proj_descr"
                  valores = valores .. ", '" .. cgi.descricao .. "'"
               end
               if cgi.setor ~= 'NULL' then 	  
                  campos  = campos .. ", set_cod"
                  valores = valores .. ", " .. cgi.setor
               end
               if cgi.cod_usuario and cgi.cod_usuario ~= '' then
                  campos = campos .. ", proj_cod_usuario"
                  valores = valores .. ", '" .. cgi.cod_usuario .. "'"
               end
               if cgi.plano_rec and cgi.plano_rec ~= '' then
                  campos = campos .. ", proj_plano_rec"
                  valores = valores .. ", " .. cgi.plano_rec
               end
               if cgi.class_acomp and cgi.class_acomp ~= '' then
                  campos = campos .. ", proj_class"
                  valores = valores .. ", " .. cgi.class_acomp
               end               
               EscreveBD("projeto", campos, valores)        
         
            else
               -- ----------------------------------------------------------------------------------------
               -- APRESENTA CAMPO UNIDADE                           
               -- Verifica se existem campos nulos antes de inserir BD para evitar erros no ACESSABD.LUA
               -- ----------------------------------------------------------------------------------------
               campos  = "org_cod, proj_nome, proj_prior, proj_dt_ini_prev, proj_dt_fim_prev, proj_dt_incl, unid_cod, "..
                         "pes_cod, proj_versao, stat_cod, proj_pae, proj_paeid, last_versao, proj_sigla, "..
                         "proj_resp_cad, proj_coord, proj_colab, proj_freq, proj_meta, proj_num_ote"
               valores = cgi.orgao..", '"..cgi.projeto.."', "..cgi.prioridade..", '"..dataini.."','"..datafim.."','"..
                         datacad.."',"..cgi.unidade..","..resp..", "..versao..",1,"..cgi.plano_emerg..", "..
                         cgi.plano_emerg_duto..",'"..last_versao.."', '"..proj_sigla.."',"..resp_cad..", '"..
                         cgi.coordenador.."', '"..getuser().."', "..cgi.freq..", "..cgi.metas..",'"..cgi.ot_especifica.."' "
               if cgi.descricao ~= 'NULL' then
                  campos  = campos .. ", proj_descr"
                  valores = valores .. ", '" .. cgi.descricao .."'"
               end
               if cgi.cod_usuario and cgi.cod_usuario ~= '' then
                  campos = campos .. ", proj_cod_usuario"   
                  valores = valores .. ", '" .. cgi.cod_usuario .. "'"
               end
               if cgi.plano_rec and cgi.plano_rec ~= '' then
                  campos = campos .. ", proj_plano_rec"
                  valores = valores .. ", '" .. cgi.plano_rec .. "'"
               end
               if cgi.class_acomp and cgi.class_acomp ~= '' then
                  campos = campos .. ", proj_class"
                  valores = valores .. ", '" .. cgi.class_acomp .. "'"
               end               
               EscreveBD("projeto", campos, valores)           
            end
         else
            projeto = ConsultaBD("projeto","*"," org_cod = "..cgi.orgao.." AND proj_nome = '"..cgi.projeto.."' ","proj_versao")
            -- ------------------------------------------------------------------------------------------
            -- Verifica se existem campos nulos antes de inserir no BD para evitar erros no ACESSABD.LUA  
            -- ------------------------------------------------------------------------------------------
            campos  = "org_cod, proj_nome, proj_prior, proj_dt_ini_prev, proj_dt_fim_prev, proj_dt_incl, pes_cod, "..
                      "proj_versao, stat_cod, proj_pae, proj_paeid, last_versao, proj_sigla, proj_resp_cad, "..
                      "proj_coord, proj_colab, proj_freq, proj_meta, proj_num_ote"
            valores = cgi.orgao..", '"..cgi.projeto.."', "..cgi.prioridade..", '"..dataini.."','"..datafim.."','"..
                      datacad.."', "..resp..","..versao..",1,'"..cgi.plano_emerg.."','"..cgi.plano_emerg_duto.."','"..
                      last_versao.."','"..proj_sigla.."',"..resp_cad..", '"..cgi.coordenador.."', '"..getuser()..
                      "', "..cgi.freq..", "..cgi.metas..",'"..cgi.ot_especifica.."' " 
            if cgi.descricao ~= 'NULL' then
               campos  = campos .. ", proj_descr"
               valores = valores .. ", '" .. cgi.descricao .."'"
            end                    
            if cgi.cod_usuario and cgi.cod_usuario ~= '' then
               campos = campos .. ", proj_cod_usuario"
               valores = valores .. ", '" .. cgi.cod_usuario .. "'"
            end
            if cgi.plano_rec and cgi.plano_rec ~= '' then
               campos = campos .. ", proj_plano_rec"
               valores = valores .. ", " .. cgi.plano_rec
            end
            if cgi.class_acomp and cgi.class_acomp ~= '' then
               campos = campos .. ", proj_class"
               valores = valores .. ", " .. cgi.class_acomp
            end            
            EscreveBD("projeto", campos, valores)            
        
         end      
   
         projeto = ConsultaBD("projeto","*"," org_cod = "..cgi.orgao.." AND proj_nome = '"..cgi.projeto.."' AND proj_versao = "..versao.." ","proj_versao")
         cod_proj = projeto.proj_cod
         cgi.proj_cod = projeto.proj_cod

         -- -------------------------------------------------------------------------------------
         -- OT OPERACIONAL
         -- Atualiza os dados referentes a CLASSIFICACAO, SUB e OT OPERACIONAL na tabela PROJ_OT
         -- -------------------------------------------------------------------------------------
         cgi.ot_sub = AtualizaOtOperacional(cod_proj, cgi.classificacao, cgi.ot_class, cgi.subclassificacao, cgi.ot_subclass, cgi.ot_oper)                  

         -- ------------------------------------------------------------------
         -- Inicializa os valores dos CGIs referentes ao Custo Previsto Anual 
         -- ------------------------------------------------------------------
         InicializaCustoPrevistoAnual()         
         
         -- ----------------------------------------------------------------------------------------
         -- Esta funcao efetua as insercoes/atualizacoes na tabela CUSTO
         -- Recebe os valores referentes ao codigo do projeto e ao FLAG flag_valor_custo_dif
         -- ----------------------------------------------------------------------------------------
         AtualizaCustoPrevistoAnual(cod_proj, flag_valor_custo_dif)
          
      end

      -- ---------------------------------
      -- INSERO NA TABELA TRANSPETRO 
      -- ---------------------------------      
      sql = "SELECT * FROM transpetro WHERE proj_cod = "..cgi.proj_cod
      DBExec(sql)
      temp = DBRow()
      if temp then
         delete = "DELETE FROM transpetro WHERE proj_cod = "..cgi.proj_cod
         DBExec(delete)
         insert = "INSERT INTO transpetro (proj_cod,trans_tipo,trans_eng,trans_cbi,trans_cac,trans_cad,trans_ui,trans_prog,trans_proj) "..
                  "VALUES ("..cgi.proj_cod..",'"..troca_nil_por_vazio(cgi.tipo).."','"..troca_nil_por_vazio(cgi.engenharia).."','"..
                  troca_nil_por_vazio(cgi.cbi).."','"..troca_nil_por_vazio(cgi.cac).."','"..troca_nil_por_vazio(cgi.cad).."','"..
                  troca_nil_por_vazio(cgi.ui).."','"..troca_nil_por_vazio(cgi.programa).."','"..troca_nil_por_vazio(cgi.proj).."')"
         DBExec(insert)
      else
         insert = "INSERT INTO transpetro (proj_cod,trans_tipo,trans_eng,trans_cbi,trans_cac,trans_cad,trans_ui,trans_prog,trans_proj) "..
                  "VALUES ("..cgi.proj_cod..",'"..troca_nil_por_vazio(cgi.tipo).."','"..troca_nil_por_vazio(cgi.engenharia).."','"..
                  troca_nil_por_vazio(cgi.cbi).."','"..troca_nil_por_vazio(cgi.cac).."','"..troca_nil_por_vazio(cgi.cad).."','"..
                  troca_nil_por_vazio(cgi.ui).."','"..troca_nil_por_vazio(cgi.programa).."','"..troca_nil_por_vazio(cgi.proj).."')"
         DBExec(insert)
      end    
  
      if cgi.botao == 'Grava' or cgi.botao == 'Grava Novo Projeto' then  
         prox_pag = 'cadastra'
      elseif cgi.botao == 'Detalhar_Custo' then
         prox_pag = 'custo_previsto'   
      elseif cgi.botao == 'Atividades' then
         prox_pag = 'atividades'        
      elseif cgi.botao == 'Obs' then
         prox_pag = 'incl_obs'
      elseif cgi.botao == 'Dutos' then
         prox_pag = 'dutos'
      elseif cgi.botao == 'Formaliza' then
         prox_pag = 'formaliza'         
      end
      
      -- -------------------------------------------------------------------
      -- DELETA INFORMAES SOBRE DUTOS SE O USURIO TIVER SELECIONADO NO
      -- -------------------------------------------------------------------      
      if tonumber(cgi.plano_emerg_duto) == 0 then
         sql = "DELETE FROM dutos WHERE proj_cod = "..cgi.proj_cod
         DBExec(sql)
      end
  
      if not cgi.proj_cod and cgi.proj_cod ~= '' then
         consulta_projeto = "SELECT MAX(proj_cod) as proj_cod FROM projeto WHERE proj_sigla = '"..proj_sigla.."'"
         DBExec(consulta_projeto)
         resultado_projeto = DBRow()
         proj_cod_colab = resultado_projeto.proj_cod
      else   
         deleta_colab = "DELETE FROM colaboradores WHERE proj_cod = "..cgi.proj_cod
         DBExec(deleta_colab)
         proj_cod_colab = tonumber(cgi.proj_cod)
      end
      tab_colab = {}
      tab_colab = Separa_Variaveis(cgi.recebe_colaboradores)  
      
      recebe_colaboradores = cgi.recebe_colaboradores       

      i = 1
      while (tab_colab[i]) do
      	 -- ---------------------------------------------------------------------------------------------
      	 -- Soh insere na tabela de colaboradores se o valor nao corresponder ao coordenador do projeto
      	 -- ---------------------------------------------------------------------------------------------
      	 if ( tonumber(tab_colab[i]) ~= tonumber(retornaPessoaCod(cgi.coordenador)) ) then      	              	  	  	 	
            insert_colab = "INSERT INTO colaboradores (proj_cod, colaborador) VALUES ("..proj_cod_colab..", "..
                           "'"..tab_colab[i].."')"
            DBExec(insert_colab)
         else 
            -- ---------------------------------------------------------------------
            -- Remove o colaborador no caso colab1,COLAB,colabn
            -- ---------------------------------------------------------------------          
            recebe_colaboradores = gsub(recebe_colaboradores, tab_colab[i]..",", "")    
            -- ---------------------------------------------------------------------
            -- Remove o colaborador no caso colab1,colabn,COLAB
            -- ---------------------------------------------------------------------                      
            recebe_colaboradores = gsub(recebe_colaboradores, ","..tab_colab[i], "")         
         end                
         i = i + 1
      end
      
      cgi.recebe_colaboradores = recebe_colaboradores  
      
      if ESTADO_BD == 'aberto' then
         FechaBD()
      end        
      if prox_pag == 'custo_previsto' then
      	 -- -------------------------------------------------------------------------------------------
      	 -- Evita passagem desnecessaria de CGIs para a proxima pagina
         -- -------------------------------------------------------------------------------------------
      	 cgilua.redirect("../"..prox_pag..".html?"..cgilua.encodetable({tipo_servico = cgi.tipo_servico,              
      	                 projeto = cgi.projeto, proj_cod = cgi.proj_cod, dt_ini_prevista = cgi.dt_ini_prevista,
      	                 dt_fim_prevista = cgi.dt_fim_prevista, custo_previsto_2000 = cgi.custo_previsto_2000, 
      	                 custo_previsto_2001 = cgi.custo_previsto_2001, custo_previsto_2002 = cgi.custo_previsto_2002, 
      	                 custo_previsto_2003 = cgi.custo_previsto_2003, custo_total = cgi.custo_total,
      	                 custo_dif = flag_valor_custo_dif}))
      elseif prox_pag == 'formaliza' then
         cgilua.redirect(prox_pag..".lua?tipo_servico=edita&acao=Formaliza&cod="..cgi.proj_cod)
      else 	 
         cgilua.redirect("../"..prox_pag..".html?"..cgilua.encodecgi())
      end   
                        
   end    -- if/else cgi.msg -- 
else
   if ESTADO_BD == 'aberto' then
      FechaBD()
   end
   cgilua.redirect('../cadastra.html?'..cgilua.encodecgi())  -- VOLTA PARA A PGINA html
end
