--=========================================================================================--
--=========================================================================================--
-- Funes de data                                                                         --
--                                                                Miguel Q Moraes, 12/2001 --   
--=========================================================================================--
--=========================================================================================--


---------------------------------------------------------------------------------------------    
-- Funo que retorna o dia da semana em que cai a data passada como parmetro --------------
---------------------------------------------------------------------------------------------
-- Retorna uma string com o nome do dia da semana -------------------------------------------
----------------------------------------------------------------- Miguel Q Moraes, 12/2001 --
function fDia_Da_Semana(data)

   data = fFormata_Data_Default(data)

   local hoje = date("%w")
   local dia = 0
   local diaDaSemana
      
   dia = mod(hoje + fData_To_Dias(data), 7)

   if dia < 0 then 
      dia = dia + 7
   end

   if dia == 0 then 
      diaDaSemana = "Domingo"
   elseif dia == 1 then
      diaDaSemana = "Segunda"
   elseif dia == 2 then
      diaDaSemana = "Tera"
   elseif dia == 3 then
      diaDaSemana = "Quarta"
   elseif dia == 4 then
      diaDaSemana = "Quinta"
   elseif dia == 5 then
      diaDaSemana = "Sexta"
   else
      diaDaSemana = "Sbado"
   end
   
   return diaDaSemana
end

---------------------------------------------------------------------------------------------    
-- Funo que retorna o nmero de dias de um mes. Recebe o nmero do ms e, opcionalmente, --
-- o ano (para o caso de anos bissextos) ----------------------------------------------------
---------------------------------------------------------------------------------------------
-- Retorna um inteiro com o nmero de dias do ms -------------------------------------------
----------------------------------------------------------------- Miguel Q Moraes, 12/2001 --
function fDias_No_Mes(mes, ...)

   local nDias
   local ano = arg[1]

   if (mes == 4) or (mes == 6) or (mes == 9) or (mes == 11) then
      nDias = 30
   elseif mes == 2 then
      nDias = 28	
      if not ano then
      	 ano = tonumber(date("%Y"))
      end	
      if mod(ano, 4) == 0 then
         nDias = 29
      end
   else 
      nDias = 31
   end

   return nDias
end

---------------------------------------------------------------------------------------------    
-- Funo que retorna o nmero de dias entre as datas passadas ("aaaammdd", "aaaammdd") ou, -
-- se no for passada a segunda data, entre a data passada e a data atual -------------------
-- S aceita datas entre 1970 e 2037 --------------------------------------------------------
---------------------------------------------------------------------------------------------
-- Retorna um inteiro -----------------------------------------------------------------------
----------------------------------------------------------------- Miguel Q Moraes, 12/2001 --    
function fData_To_Dias(data, ...)

   local hoje = arg[1]
 
   if arg[1] then
      hoje = fFormata_Data(arg[1])
   else	
      hoje = date("%Y/%m/%d")
   end

   data = fFormata_Data(data)
     
   secStart = date2sec(hoje, "%Y/%m/%d")
   secEnd = date2sec(data, "%Y/%m/%d")
   
   local secs = secEnd - secStart
  
   if secs < 0 then
      secs = - secs
   end
   
   dias = secs / 86400
  
   return dias
end

---------------------------------------------------------------------------------------------    
-- Funo que retorna o nmero de dias entre as datas passadas ("aaaammdd", "aaaammdd") ou, -
-- se no for passada a segunda data, entre a data passada e a data atual -------------------
-- Igual a funo fData_To_Dias mas aceita qualquer data ------------------------------------
---------------------------------------------------------------------------------------------
-- Retorna um inteiro -----------------------------------------------------------------------
----------------------------------------------------------------- Miguel Q Moraes, 12/2001 --    
function fData_To_Dias_Ex(data, ...)

   local hoje = arg[1]

   if arg[1] then
      hoje = fFormata_Data_Default(arg[1])
   else	
      hoje = date("%Y%m%d")
   end
   
   data = fFormata_Data_Default(data)
   
   local dias = 0
   local minus = 1
   
   local ano = tonumber(strsub(data, 1, 4))
   local mes = tonumber(strsub(data, 5, 6))
   local dia = tonumber(strsub(data, 7, 8))

   diasNoMes = fDias_No_Mes(mes, ano)

   if format("%04d%02d%02d", ano, mes, dia) < hoje then
      local temp = hoje
      hoje = format("%04d%02d%02d", ano, mes, dia)
      ano = tonumber(strsub(temp, 1, 4))
      mes = tonumber(strsub(temp, 5, 6))
      dia = tonumber(strsub(temp, 7, 8))
      minus = -1
   end
  
   while format("%04d%02d%02d", ano, mes, dia) ~= hoje do
      dia = dia - 1
      if dia < 1 then
         mes = mes - 1
      	 if mes < 1 then
   	    mes = 12
   	    ano = ano - 1
 	 end
         dia = fDias_No_Mes(mes, ano)
      end
      dias = dias + 1
   end
   
   return (minus * dias)
end

---------------------------------------------------------------------------------------------    
-- Funo que retorna a data "diasTotal" dias depois da data passada como segundo parmetro -
-- Se no for passado o segundo parmetro, a data de hoje  usadada -------------------------
---------------------------------------------------------------------------------------------
-- Retorna uma string com a data no formato padro "aaaammdd" -------------------------------
----------------------------------------------------------------- Miguel Q Moraes, 12/2001 --    
function fDias_To_Data(diasTotal, ...)

   local datahoje

   if arg[1] then
      datahoje = fFormata_Data(arg[1])
   else
      datahoje = date("%d/%m/%Y")
   end
   
   secs = diasTotal * 86400

   local hojeSecs = date2sec(datahoje, "%Y/%m/%d")
   
   return fFormata_Data_Default(sec2date(hojeSecs + secs, "%d/%m/%Y"))
end

---------------------------------------------------------------------------------------------    
-- Funo que retorna a data "diasTotal" dias depois da data passada como segundo parmetro -
-- Se no for passado o segundo parmetro, a data de hoje  usadada -------------------------
---------------------------------------------------------------------------------------------
-- Retorna uma string com a data no formato padro "aaaammdd" -------------------------------
----------------------------------------------------------------- Miguel Q Moraes, 12/2001 --    
function fDias_To_Data_Ex(diasTotal, ...)
   
   local datahoje
   
   if arg[1] then
      datahoje = fFormata_Data_Default(arg[1])
   else
      datahoje = date("%Y%m%d")
   end

   local anoCorrente = tonumber(strsub(datahoje, 1, 4))
   local mesCorrente = tonumber(strsub(datahoje, 5, 6))
   local diaCorrente = tonumber(strsub(datahoje, 7, 8))
   
   diasNoMes = fDias_No_Mes(mesCorrente, anoCorrente)

   while diasTotal > (diasNoMes - diaCorrente) do
	mesCorrente = mesCorrente + 1
   	if mesCorrente > 12 then
   	   mesCorrente = 1
   	   anoCorrente = anoCorrente + 1 
   	end
   	diasTotal = diasTotal - (diasNoMes - diaCorrente)
   	diaCorrente = 0
   	diasNoMes = fDias_No_Mes(mesCorrente, anoCorrente)
   end
   
   diaCorrente = diaCorrente + diasTotal

   return format("%04d%02d%02d", anoCorrente, mesCorrente, diaCorrente)
end 

---------------------------------------------------------------------------------------------    
-- Funo que retorna a data daqui a "semanas" semanas --------------------------------------
---------------------------------------------------------------------------------------------
-- Retorna uma string com a data no formato padro "aaaammdd" -------------------------------
----------------------------------------------------------------- Miguel Q Moraes, 12/2001 --
function fSemanas_To_Data(semanas)

   return fDias_To_Data(semanas * 7)
end

---------------------------------------------------------------------------------------------    
-- Funo para validar data passada no parmetro "data", passado no formato "aaaammdd" ------
----------------------------------------------------------------------------------- Reviso -
-- Retorna a data ("aaaammdd") ou "Invlido" ------------------------------------------------
----------------------------------------------------------------- Miguel Q Moraes, 12/2001 --
function fCheca_Data(data)

   local check = 'OK'

   if data ~= '' then
     local ano = tonumber(strsub (data,1,4))
     local mes = tonumber(strsub (data,5,6))
     local dia = tonumber(strsub (data,7,8))
     
     if not dia or (dia < 1 or dia > fDias_No_Mes(mes, ano)) then
        check = 'Dia Invlido'
     end
 
     if not mes or (mes < 1 or mes > 12) then
        check = 'Ms Invlido'
     end  

     if not ano or (ano < 1900 or ano > 2500) then
        check = 'Ano Invlido'
     end
   else
      return nil
   end
   
   return check
end

---------------------------------------------------------------------------------------------    
-- Funo que recebe uma data passada nos formatos "ddmmaaaa", "aaaammdd", "dd/mm/aaaa" ou --
-- "aaaa/mm/dd" e a retorna no formato default "aaaammdd" e como segundo, terceiro e quarto -
-- valores de retorno, o ano, o mes e o dia, respectivamente. Se um segundo parmetro for ---
-- passado, a ordem de retorno muda para "ddmmaaaa", dia, mes, ano --------------------------
---------------------------------------------------------------------------------------------
-- Retorna quatro strings -------------------------------------------------------------------
----------------------------------------------------------------- Miguel Q Moraes, 12/2001 --
function fFormata_Data_Default(data, ...)

   local dia = ''
   local mes = ''
   local ano = ''
   local check
    
   if data and data ~= '' then
      data = gsub(data, '/', '')	
      
      dia = strsub (data, 1, 2)
      mes = strsub (data, 3, 4)
      ano = strsub (data, 5, strlen(data))
      
      check = fCheca_Data(ano..mes..dia)
      
      if (check ~= 'OK') and (check ~= 'Dia Invlido') then
         ano = strsub (data, 1, 4)
         mes = strsub (data, 5, 6)
         dia = strsub (data, 7, 8)
         check = fCheca_Data(ano..mes..dia)
      end

      if check ~= 'OK' then
      	 return check
      end      
   else
      return nil
   end
   
   if arg[1] then
      return dia..mes..ano, dia, mes, ano, check
   else
      return ano..mes..dia, ano, mes, dia, check
   end   
end

---------------------------------------------------------------------------------------------    
-- Funo que recebe uma data passada nos formatos "ddmmaaaa", "aaaammdd", "dd/mm/aaaa" ou --
-- "aaaa/mm/dd" e a retorna no formato "dd/mm/aaaa" e, como segundo valor de retorno, o dia -
-- da semana. Se um segundo parmetro opcional for passado, a ordem da data de retorno se ---
-- inverte para "aaaa/mm/dd" ----------------------------------------------------------------
---------------------------------------------------------------------------------------------
-- Retorna uma string com a data formatada --------------------------------------------------
----------------------------------------------------------------- Miguel Q Moraes, 12/2001 --
function fFormata_Data(data, ...)

   local temp, dia, mes, ano = fFormata_Data_Default(data, arg[1])

   if dia and mes and ano then
      return dia..'/'..mes..'/'..ano
   end
end