-- ---------------------------------------------------------------------------------------
-- permissao_acesso.lua
-- ---------------------------------------------------------------------------------------
-- Usa funcoes da GLOBAL.LUA
-- ---------------------------------------------------------------------------------------
-- CONJUNTO DE FUNCOES QUE VERIFICAM O NIVEL DE ACESSO PERMITIDO AO USUARIO NAS FUNCOES
-- DE CONSULTA/RELATORIO E CADASTRAMENTO/EDICAO
-- ---------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------
-- Verifica se o usuario (getuser()) passado como parametro tem acesso total as consultas
-- e relatorios ( _CHAVES_ACESSO_COMPLETO_ )
-- Se tiver retorna "1", caso contrario retorna nil
-- ---------------------------------------------------------------------------------------
   function acessoCompleto( usuario )
      if not (temConteudo( usuario )) then
         error('No foi especificado o login do usurio!')
      end
           
      dir = pathLua()        
      dofile( dir..'constantes.lua' )
  
      local campo = next( _CHAVES_ACESSO_COMPLETO_, nil )     -- captura primeiro campo da tabela      
      while campo do               
         if strupper(campo) == strupper(usuario) then 
            return (1)
         end     
         campo = next( _CHAVES_ACESSO_COMPLETO_, campo )      -- captura proximo campo da tabela
      end     
               
      return nil
   end   
   
   
-- ---------------------------------------------------------------------------------------
-- Verifica se o usuario (getuser()) passado como parametro eh responsavel por alguma AN
-- ---------------------------------------------------------------------------------------
   function gerenteAN( usuario )
      if not (temConteudo( usuario )) then
         error('No foi especificado o login do usurio!')
      end
      
      if ESTADO_BD ~= 'aberto' then
         AbreBD()
      end      
      
      local sql = "SELECT org_cod, org_sigla FROM orgao o, pessoal p WHERE p.pes_chave = '"..
                   usuario.."' AND (p.pes_cod = o.pes_cod_resp OR p.pes_cod = o.pes_cod_sub)"       
      DBExec(sql)             
      local resp = DBRow()
      
      if resp then
         local org_cod   = '' 
         local org_sigla = ''
         local tab = {} 
         
         local i = 0
         while resp do
            unid_cod     = insereLista(unid_cod, resp.unid_cod, ',')
            unid_sigla   = insereLista(unid_sigla, resp.unid_sigla, ', ')                     
            
            org_cod      = insereLista(org_cod, resp.org_cod, ',')
            org_sigla    = insereLista(org_sigla, resp.org_sigla, ', ')
            
            tab          = insereTabela(tab, resp.org_cod, 0, ',')                          
            
            resp = DBRow() 
            i    = i + 1 
         end                     
         
         -- ------------------------------------------------------------------------------------------
         -- PARAMETROS retornados: tem area de negocio (1), numero de areas gerenciadas pelo usuario,
         --                        codigo da area, sigla da area e uma tabela contendo os codigos
         --                        das UNs indexados pelos codigos das ANs (associa cada UN a sua AN)
         --                        como o usuario eh gerente da AN tem acesso a todas as UNs (0)  
         -- ------------------------------------------------------------------------------------------
         return 1, i, org_cod, org_sigla, tab
      else               
         return nil
      end   
   end   

-- ---------------------------------------------------------------------------------------
-- Verifica se o usuario (getuser()) passado como parametro eh responsavel por alguma UN
-- ---------------------------------------------------------------------------------------
   function gerenteUN( usuario )
      if not (temConteudo( usuario )) then
         error('No foi especificado o login do usurio!')
      end
      
      if ESTADO_BD ~= 'aberto' then
         AbreBD()
      end      
      
      local sql = "SELECT u.pes_cod_resp, u.pes_cod_sub, u.unid_cod, u.unid_sigla, o.org_cod, o.org_sigla "..
                  "FROM unidade u, orgao o, pessoal p WHERE p.pes_chave = '"..usuario.."' AND o.org_cod = "..
                  "u.org_cod AND (p.pes_cod = u.pes_cod_resp OR p.pes_cod = u.pes_cod_sub)"                                           
      DBExec(sql)             
      local resp = DBRow()
      
      if resp then
         local unid_cod     = '' 
         local unid_sigla   = ''
         local unid_org_cod = ''
         local unid_org_sig = ''
         local tab = {}
         
         local i = 0
         while resp do          
            unid_cod     = insereLista(unid_cod, resp.unid_cod, ',')
            unid_sigla   = insereLista(unid_sigla, resp.unid_sigla, ', ')
            
            unid_org_cod = insereLista(unid_org_cod, resp.org_cod, ',')                     
            unid_org_sig = insereLista(unid_org_sig, resp.org_sigla, ', ')
            
            tab          = insereTabela(tab, resp.org_cod, resp.unid_cod, ',')                       
            
            resp = DBRow()   
            i    = i + 1 
         end         
         
         -- -----------------------------------------------------------------------------------------------------
         -- PARAMETROS retornados: tem unidade de negocio (1), numero de unidades gerenciadas pelo usuario,
         --                        codigo da unidade, sigla da unidade, codigo do orgao e uma tabela contendo os
         --                        codigos das UNs indexados pelos codigos das ANs (associa cada UN a sua AN)  
         -- -----------------------------------------------------------------------------------------------------                         
         return 1, i, unid_cod, unid_sigla, unid_org_cod, unid_org_sig, tab
      else               
         return nil
      end   
   end   


-- ---------------------------------------------------------------------------------------
-- Verifica a permissao de acesso do usuario para ver projetos em CONSULTAS/RELATORIOS
-- e CADASTRAR/EDITAR projetos
-- ---------------------------------------------------------------------------------------
-- LOGICA DE FUNCIONAMENTO
-- ---------------------------------------------------------------------------------------
-- 1. Verifica se o usuario tem acesso completo aos projetos do sistema (constante
--    _CHAVES_ACESSO_COMPLETO_ do script CONSTANTES.LUA) -> funcao acessoCompleto()
-- 2. Verifica se o usuario eh GERENTE de alguma(s) AN(s) (os campos PES_COD_RESP e 
--    PES_COD_SUB da tabela ORGAO) -> gerenteAN()
-- 3. Verifica se o usuario eh GERENTE de alguma(s) UN(s) (os campos PES_COD_RESP e 
--    PES_COD_SUB da tabela UNIDADE) -> gerenteUN()
-- 4. Na tabela de PESSOAL
--    4.1 Verifica se o usuario tem acesso a uma determinada AN (PES_ACESSO_ORG)
--    4.2 Verifica se o usuario tem acesso a uma determinada UN (PES_UNID)  
-- ---------------------------------------------------------------------------------------
-- OBS sobre os campos PES_ACESSO_ORG e PES_UNID:
-- ---------------------------------------------------------------------------------------
-- Todo usuario deve ter cadastrado na tabela PESSOAL o seu PES_UNID (a menos que ele 
-- tenha acesso completo ou seja GERENTE de alguma(s) AN(s) e/ou UN(s)
-- Caso o usuario nao seja GERENTE de AN mas se deseja dar acesso a esse usuario a todas
-- as UNs dessa AN, entao cadastra-se o campo PES_ACESSA_ORG
-- ---------------------------------------------------------------------------------------
-- OBS: O campo PES_ORG da tabela de pessoal soh estah sendo utilizado atualmente para 
--      recuperar os colaboradores em CADASTRO/EDIO e se refere a AN do usuario
-- ---------------------------------------------------------------------------------------
function validaAcesso()
   local usuario      = getuser()
   local tab_aux_org  = {}   
   local tab_aux_unid = {} 
   local associacao   = {}
   local campo, valor
   local ehGerenteAN, num_org, org_cod, org_sigla
   local ehGerenteUN, num_unid, unid_cod, unid_sigla, unid_org_cod, unid_org_sig
   local flag_org, flag_unid, org_cod, org_sigla, unid_cod, unid_sigla            
      
   if ESTADO_BD ~= 'aberto' then
      AbreBD()
   end 
    
   -- -----------------------------------------------------------------------------------
   -- Verifica ACESSO COMPLETO            
   -- -----------------------------------------------------------------------------------
   if not acessoCompleto(usuario) then          
      -- -----------------------------------------------------------------------------------
      -- Verifica GERENTE AN          
      -- -----------------------------------------------------------------------------------     
      -- Se o usuario eh responsavel por alguma AN os valores para org_cod, org_sigla e 
      -- flag_org sao atualizados e o usuario passa a ter acesso a todas as unidades da AN
      -- -----------------------------------------------------------------------------------
      ehGerenteAN, num_org, org_cod, org_sigla, tab_aux_org = gerenteAN(usuario) 
      if ehGerenteAN then         
      	 associacao = tab_aux_org 
      	 -- como o usuario eh gerente AN ele tem acesso a todas as UNs da AN, logo 
      	 flag_unid  = 2 
      	 -- atualiza inicialmente o numero de ANs associadas ao usuario     	 
         flag_org  = atualizaFlag(flag_org, num_org)     	
      end                                            
      
      -- -----------------------------------------------------------------------------------
      -- Verifica GERENTE UN          
      -- -----------------------------------------------------------------------------------
      -- Se o usuario eh responsavel por alguma UN os valores para unid_cod, unid_sigla e flag_unid sao
      -- atualizados e o usuario passa a ter acesso a todas as unidades em que ele eh gerente
      ehGerenteUN, num_unid, unid_cod, unid_sigla, unid_org_cod, unid_org_sig, tab_aux_unid = gerenteUN(usuario)
      if ehGerenteUN then           
         -- -----------------------------------------------------------------------------------
         -- Faz atualizacoes em variaveis de acordo com o resultado de GERENTE AN / GERENTE UN
         -- -----------------------------------------------------------------------------------           
         -- Se o usuario eh gerente de AN e de UN, concatena as tabelas que associam 
         -- informacoes sobre as ANs das ANs e UNs
         -- -----------------------------------------------------------------------------------
         if ehGerenteAN then        
            campo, valor = next( tab_aux_unid, nil )      -- captura primeiro campo da tabela      
            while campo do                             
               associacao   = insereTabela(associacao, campo, valor, ',') 
               org_cod      = insereLista(org_cod, campo, ',')             
               campo, valor = next( tab_aux_unid, campo )       -- captura proximo campo da tabela
            end 
                
            -- Adiciona um separador ao final da string para o IF interno tratar todas as ANs
            unid_org_sig = unid_org_sig..', '         
            while temConteudo(unid_org_sig) do
               ini, fim = strfind(unid_org_sig,', ') 
               if ini then
                  org_sig      = insereLista(org_sig, strsub(unid_org_sig, 1, ini-1), ', ')
                  unid_org_sig = strsub(unid_org_sig, fim + 1)               
               end                 
               org_sigla  = insereLista(org_sigla, org_sig, ', ')            
            end             
         -- Atribui a tabela que associam ANs e UNs (associacao) a tabela retornada por GerenteUN (tab_aux_unid) 
         else 
            associacao = tab_aux_unid 
            org_cod    = unid_org_cod
            org_sigla  = unid_org_sig
            -- atualiza inicialmente o numero de UNs associadas ao usuario     	                        
            flag_unid = atualizaFlag(flag_unid, num_unid) 
         end 
      end   
             
      -- -----------------------------------------------------------------------------------
      -- Verifica PES_ACESSO_ORG          
      -- -----------------------------------------------------------------------------------     
      -- Verifica AN e UN do usuario na tabela de PESSOAL
      sql_AN = "SELECT pes_acesso_org, org_sigla FROM pessoal p, orgao o "..
               "WHERE pes_chave = '" .. usuario .. "' AND p.pes_acesso_org *= o.org_cod "
      DBExec(sql_AN)
      resultadoAN = DBRow()                                                                                   
         
      if resultadoAN and resultadoAN.pes_acesso_org then 
         -- Acumula a AN do usuario  
            org_cod   = insereLista(org_cod, resultadoAN.pes_acesso_org, ',')
            org_sigla = insereLista(org_sigla, resultadoAN.org_sigla , ', ')
             
         -- tabela responsavel pela associacao das ANs e UNs na tabela de PESSOAL
            associacao = insereTabela(associacao, resultadoAN.pes_acesso_org, 0, ',')
         
         -- como o usuario tem acesso a uma AN na tabela de PESSOAL, ele tem acesso a todas
         -- as UNs da AN, logo
            flag_unid = 2 
      end
      
      -- -----------------------------------------------------------------------------------
      -- Verifica PES_UNID        
      -- ----------------------------------------------------------------------------------- 
      sql_UN = "SELECT unid_cod, unid_sigla, o.org_cod, o.org_sigla FROM orgao o, unidade u "..
               "WHERE unid_cod IN (select pes_unid from pessoal where pes_chave = '" .. usuario .. "') "..
               "AND o.org_cod = u.org_cod "                           
      DBExec(sql_UN)
      resultadoUN = DBRow() 
      
      if resultadoUN and resultadoUN.unid_cod then  
         if ( not( temConteudo(unid_cod) ) ) or 
            ( unid_cod and not( strfind(','..unid_cod..',', ','..resultadoUN.unid_cod..',') ) ) then                        
            flag_unid = atualizaFlag(flag_unid) 
         end                                                                                                                                                           
                                         
         -- Acumula a UN do usuario                
            unid_cod   = insereLista(unid_cod, resultadoUN.unid_cod, ',')                  
            unid_sigla = insereLista(unid_sigla, resultadoUN.unid_sigla, ', ') 
         
         -- Acumula a AN do usuario  
            org_cod   = insereLista(org_cod, resultadoUN.org_cod, ',')             
            org_sigla = insereLista(org_sigla, resultadoUN.org_sigla , ', ')  
            
         -- tabela responsavel pela associacao das ANs e UNs na tabela de PESSOAL
            associacao = insereTabela(associacao, resultadoUN.org_cod, resultadoUN.unid_cod, ',')
      end             
         
      -- Trata quando nao temos nem AN nem UN cadastradas na tabela de PESSOAL
      if ( not(resultadoAN) and not(resultadoUN) and not(ehGerenteAN) and not(ehGerenteUN) ) then                                     
         -- O usuario nao possui nem AN nem UN cadastradas na tabela de pessoal
         -- e nem eh gerente de AN e/ou UN                    
            flag_sem_acesso = 1  
      end   
      
      -- Pega o tamanho da tabela que corresponde ao numero de ANs relacionadas ao usuario para
      -- atualizar o valor de FLAG_ORG que informa o numero de ANs associadas a ele            
      if associacao then
         i = 0
         campo = next( associacao, nil )            -- captura primeiro campo da tabela      
         while campo do                                 
            campo = next( associacao, campo )       -- captura proximo campo da tabela
            i = i + 1                
         end 
         
         flag_org = atualizaFlag(flag_org, i)
      end     
                                
      -- Se o usuario nao tem permissao de acesso redireciona para uma pagina de ALERTA    
      if ( flag_sem_acesso  == 1 ) or 
         ( not( temConteudo(flag_org) ) and not( temConteudo(flag_unid) ) ) then      
         cgilua.includehtml("sem_acesso.html")   -- Inclui na pagina codigo HTML de sem_acesso.html      
         exit()                                  -- Evita que o restante do codigo seja executado
      end               
 
      -- ----------------------------------------------------------------------------------------------
      -- retorna flags relacionados a orgao e unidade indicando a permissao de acesso do usuario
      -- ----------------------------------------------------------------------------------------------
      -- Se usuario nao tem ACESSO COMPLETO, entao retorna:
      -- ----------------------------------------------------------------------------------------------
      -- FLAG_ORG:      retorna 1 se o usuario estah associado a SOMENTE uma AN, 2 se o usuario estah
      --                associado a MAIS de uma AN e NIL se o usuario nao estah associado a nenhuma AN 
      -- FLAG_UNID:     retorna 1 se o usuario estah associado a SOMENTE uma UN, 2 se o usuario estah
      --                associado a MAIS de uma UN e NIL se o usuario nao estah associado a nenhuma UN 
      -- ORG_COD:       retorna o(s) codigo(s) do(s) orgao(s) ao(s) qual(is) o usuario tem acesso
      -- ORG_SIGLA:     retorna a(s) sigla(s) do(s) orgao(s) ao(s) qual(is) o usuario tem acesso
      -- UNID_COD:      retorna o(s) codigo(s) da(s) unidade(s) a(s) qual(is) o usuario tem acesso
      -- UNID_SIGLA:    retorna a(s) sigla(s) da(s) unidade(s) a(s) qual(is) o usuario tem acesso
      -- ASSOCIACAO:    retorna uma tabela com as UNs associadas as suas ANs (EX: AN:5 - UN: 8,11,12
      --                gera TAB[5] = '8, 11, 12')
      -- ---------------------------------------------------------------------------------------------       
      -- Se o usuario tem acesso completo, NIL eh retornado para todos os campos
      -- ---------------------------------------------------------------------------------------------       
      return flag_org, flag_unid, org_cod, org_sigla, unid_cod, unid_sigla, associacao          
   
   else -- if not acessoCompleto(usuario)
      return nil, nil, nil, nil, nil, nil, nil  
   end  
    
end

-- ---------------------------------------------------------------------------------------
-- << Funcoes auxiliares da funcao validaAcesso() 
-- ---------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------
-- Insere um VALOR em uma LISTA separando esses elementos pelo SEPARADOR passado com
-- parametro e verificando se o VALOR jah nao consta na LISTA, evitando repeticoes
-- ---------------------------------------------------------------------------------------
function insereLista(lista, valor, separador)
   if temConteudo( lista ) and temConteudo( valor ) then
      -- ---------------------------------------------------------------------------------
      -- Estao sendo inseridos tanto os codigos quanto as siglas das ANs e UNs
      -- Em algumas siglas aparece o caracter '-' que eh caracter reservado no STRFIND
      -- Esse caracter eh substituido por '' para evitar problemas de reconhecimento
      -- ---------------------------------------------------------------------------------
      if not( strfind(separador..gsub(lista,'-','')..separador, 
         separador..gsub(valor,'-','')..separador) ) then              
         lista = lista..separador..valor
      end
   elseif temConteudo( valor ) then
      lista = valor
   end      
      
   return lista
end


-- --------------------------------------------------------------------------------------
-- Insere na tabela as ANs associadas as UNs controlando o nivel de visualizacao de
-- projetos pelo usuario (se ele pode ver todas as UNs '0' ou se ele tem acesso restrito
-- a algumas ANs
-- Essa funcao evita repeticoes de insercoes na tabela
-- --------------------------------------------------------------------------------------
function insereTabela(tabela, indice, valor, separador)      
   if not( tabela[indice] ) or valor == 0 then
      tabela[indice] = valor
   elseif ( tabela[indice] and tabela[indice] ~= 0 and 
      not(strfind(separador..tabela[indice]..separador, separador..valor..separador)) ) then
      tabela[indice] = tabela[indice]..separador..valor   
   end  
   return tabela
end

      
-- ---------------------------------------------------------------------------------------
-- Retorna o diretorio em que os SCRIPTS LUA devem ser chamados
-- ---------------------------------------------------------------------------------------
function pathLua()
   if ( strfind(cgilua.script_vdir, '/lua') ) then
       reldir = ''
   else
       reldir = 'lua/'
   end 
   return reldir    
end     

-- ---------------------------------------------------------------------------------------
-- Atualiza FLAG_ORG e FLAG_UNID
-- ---------------------------------------------------------------------------------------
function atualizaFlag(flag, incremento)
   if temConteudo( flag ) then
      flag = 2
   else
      if temConteudo( incremento ) then
         if incremento == 1 then
            flag = 1 
         elseif incremento > 1 then  
            flag = 2
         end
      else
         flag = 1
      end          
   end     
   return flag    
end       
      
-- ---------------------------------------------------------------------------------------
-- Funcoes auxiliares da funcao validaAcesso() >>
-- ---------------------------------------------------------------------------------------


-- ---------------------------------------------------------------------------------------
-- Funcoes auxiliar para criar condicao aninhada referente a ANs e UNs nas queries feitas
-- em CONSULTA e RELATORIO
-- ---------------------------------------------------------------------------------------
-- ---------------------------------------------------------------------------------------
-- Montagem de condicoes em consultas de AN e UN avaliando a tabela ASSOCIACAO e se
-- alguma AN foi selecionada
-- ---------------------------------------------------------------------------------------
function montaCondicao( associacao, flag_org, flag_unid, org_cod, unid_cod, id_tabela, org_sel, unid_sel )
   local cond   
   if associacao then
      local campo, valor, org, org0
      
      campo, valor = next( associacao, nil )            -- captura primeiro campo da tabela           
      while campo do                               
         if valor == 0 then                             -- tem acesso a todas as unidades do orgao
            -- Se alguma AN (org_sel) foi selecionada no form tem que ser feita restricao na AN 
            if not org_sel or ( org_sel and strfind(','..org_sel..',',','..campo..',') ) then
               valor = verificaUnidSel0(unid_sel, campo, ',')                               
               if temConteudo(valor) and valor ~= 0 then  
                  cond  = insereCond(cond,'('..id_tabela..'.org_cod = '..campo..' AND '..id_tabela..'.unid_cod IN ('..valor..'))', ' OR ')        
               else   
                  cond  = insereCond(cond,'('..id_tabela..'.org_cod = '..campo..')', ' OR ')        
               end               
            end             
         else                                           -- tem acesso restrito as unidades do orgao
            -- Se alguma AN (org_sel) foi selecionada no form tem que ser feita restricao na AN 
            if not org_sel or ( org_sel and strfind(','..org_sel..',',','..campo..',') ) then
               valor = verificaUnidSel(unid_sel, valor, ',')                               
               cond  = insereCond(cond,'('..id_tabela..'.org_cod = '..campo..' AND '..id_tabela..'.unid_cod IN ('..valor..'))', ' OR ')                 
            end         
         end                   
            
         campo, valor = next( associacao, campo )       -- captura proximo campo da tabela
      end
      
      if temConteudo(cond) then     
         cond = '( '..cond..' )'                                         
      end   
   end     
   
   return cond
end

-- Verifica se as unidades em ASSOCIACAO constam na lista de unidades que foram selecionadas
function verificaUnidSel(unid_sel, unid, separador)
   local unidade, unidades, i
   unidade = ''
   
   if temConteudo(unid_sel) then
      unidades = Separa_Variaveis(unid)
      i = 1
      while unidades[i] do
         if strfind(separador..unid_sel..separador, separador..unidades[i]..separador) then
            unidade = insereLista(unidade, unidades[i], separador) 
         end
         i = i + 1
      end 
      return unidade              
   end 
   
   return unid
end

-- Verifica se alguma unidade do orgao consta na lista de unidades que foram selecionadas
function verificaUnidSel0(unid_sel, org, separador)
   local unidade, unid  
   unidade = ''   
  
   if temConteudo(unid_sel) then      
      DBExec("SELECT unid_cod FROM unidade WHERE org_cod= ".. org)
      unid = DBRow()        
      while unid do
         if strfind(separador..unid_sel..separador, separador..unid.unid_cod..separador) then
            unidade = insereLista(unidade, unid.unid_cod, separador) 
         end
         unid = DBRow()
      end  
      return unidade               
   end 
   
   return 0
end

-- ---------------------------------------------------------------------------------------
-- Insere uma condicao em uma lista para um WHERE de uma QUERY
-- ---------------------------------------------------------------------------------------
function insereCond(lista, valor, separador)
   if temConteudo( lista ) and temConteudo( valor ) then                 
      lista = lista..separador..valor
   elseif temConteudo( valor ) then
      lista = valor
   end      
      
   return lista
end