import java.awt.*;
import java.net.*;

class TimeBar {
  // implementation
  int startPos;
  int endPos;
  Color color;
}

class PlainTimeBar extends TimeBar {
}

public class Task {
  // implementation
  String caption;
  Link caption_link;
  String status;
  PlainTimeBar estimate;
  PlainTimeBar current;
  Link current_link;

  // valore para status -- indicado pelo usuario
  final String NAO_INICIADO = "marca";
  final String EM_ALERTA    = "alerta";
  final String ATRASADO     = "atrasado";
  final String CONCLUIDO    = "concluido";
  final String EM_ANDAMENTO = "em andamento";
  final String VAZIO        = "nada";

  Task()
  {
    caption = null;
    caption_link = new Link();
    status = null;
    estimate = new PlainTimeBar();
    current = new PlainTimeBar();
    current_link = new Link();
  }

  // Caption ......................................................

  public void setCaption(String caption)
  {
    this.caption = caption;
  }

  public String getCaption()
  {
    return caption;
  }

  public void setCaptionURL(String url)
  {
    caption_link.setURL(url);
  }

  public String getCaptionURL()
  {
    return caption_link.getURL();
  }

  public Link getCaptionLink()
  {
    return caption_link;
  }

  public void setCaptionFrame(String target)
  {
    caption_link.setFrame(target);
  }

  public String getCaptionFrame()
  {
    return caption_link.getFrame();
  }

  // Current Date ...................................................

  public void setCurrentURL(String url)
  {
    current_link.setURL(url);
  }

  public String getCurrentURL()
  {
    return current_link.getURL();
  }

  public Link getCurrentLink()
  {
    return current_link;
  }

  public void setCurrentFrame(String target)
  {
    current_link.setFrame(target);
  }

  public String getCurrentFrame()
  {
    return current_link.getFrame();
  }

  public void setStatus(String status)
  {
    this.status = status;
  }

  public void setEstimatedStart(int start)
  {
    this.estimate.startPos = start;
  } 

  public void setEstimatedEnd(int end)
  {
    this.estimate.endPos = end;
  } 

  public void setCurrentStart(int start)
  {
    this.current.startPos = start;
  } 

  public void setCurrentEnd(int end)
  {
    this.current.endPos = end;
  }

  public int getCurrentEnd()
  {
    return(this.current.endPos);
  }

  public void drawBars(Graphics g, int y, int height, CoordinateMapper mapper,
                       HotSpot hotspots)
  {
    Color barColor;
    int xpos, ypos;
    int upperMargin, barWidth, barHeight;
    Point p;
    Bar bar;
    Diamond diamond;

    // no mapper
    if(mapper==null)
      return;

    upperMargin = 14;
    barHeight = 10;

    // estimate
    xpos = mapper.user2Screen(estimate.startPos);
    ypos = y + upperMargin; 
    p = new Point(xpos, ypos);
    barWidth = mapper.user2Screen(estimate.endPos) - mapper.user2Screen(estimate.startPos);
    barColor = CustomColor.x666666;
    bar = new Bar(p, barWidth, barHeight, barColor); 
    bar.draw(g);

    // current
    // Data de hoje < Data de startPos
    if(status.equalsIgnoreCase(VAZIO))
      return;

    if(status.equalsIgnoreCase(NAO_INICIADO)) {
      ypos = y + height - upperMargin - barHeight;
      p = new Point(xpos, ypos);
      diamond = new Diamond(p);
      diamond.setAlign(0); // center
      diamond.draw(g);

      // add hotspot
      if(hotspots != null) {
        hotspots.add(diamond.x(),diamond.y(),diamond.width(),diamond.height(), 
                     this.getCurrentLink());
      }
    }
    else {
      xpos = mapper.user2Screen(current.startPos);
      ypos = y + height - upperMargin - barHeight;
      p = new Point(xpos, ypos);
      barWidth = mapper.user2Screen(current.endPos) - mapper.user2Screen(current.startPos);
      barColor = status2Color();
      bar = new Bar(p, barWidth, barHeight, barColor); 
      bar.draw(g);
      if(hotspots != null) {
        hotspots.add(xpos,ypos,barWidth,barHeight, this.getCurrentLink());
      }
    }
  }

  private Color status2Color()
  {
    if(status.equalsIgnoreCase(ATRASADO))
      return CustomColor.xFF3300;
    else if(status.equalsIgnoreCase(EM_ALERTA))
      return CustomColor.xFFCC00;
    else if(status.equalsIgnoreCase(CONCLUIDO))
      return CustomColor.x009900;
    else if(status.equalsIgnoreCase(EM_ANDAMENTO))
      return CustomColor.x99CCFF;
    else // default
      return CustomColor.x666666;
  }
 
}
