import java.util.Vector;
import java.awt.*;


// ----------------------------------------------------------------------- //

class CommandTypeConstant {
  // constants
  public final static int LABEL = 1;
  public final static int SPACING = 2;
}

// ----------------------------------------------------------------------- //

class CommandType {
  int type;

  CommandType(int type)
  {
    if(type!=CommandTypeConstant.LABEL && 
       type!=CommandTypeConstant.SPACING) {
      System.out.println("CommandType error: unknown type");
      return;
    }

    this.type = type;
  }

  public int type()
  {
    return(this.type);
  }
}

// ----------------------------------------------------------------------- //

class CmdMeasure {
  int x;
  int y;
  int height;
  int width;

  // constructor
  CmdMeasure(String label, int size, int style, String type)
  {
    Abc abc = new Abc(label,size,style,type);
    width = abc.width();
    height = abc.ascent();
  }

  void setPosition(int x0, int y0)
  {
    this.x = x0;
    this.y = y0;
  }

  int x() {
    return this.x;
  }

  int y() {
    return this.y;
  }

  int width() {
    return this.width;
  }

  int height() {
    return this.height;
  }
}

// ----------------------------------------------------------------------- //

class CmdLabel {
  String label;
  int size;
  Color color;
  int style;
  String type;

  // constructor
  CmdLabel(String label, int size, Color color, int style, String type)
  {
    this.label = label;
    this.size = size;
    this.color = color;
    this.style = style;
    this.type = type;
  }

  public String label()
  {
    return(this.label);
  }

  public int size()
  {
    return(this.size);
  }

  public Color color()
  {
    return(this.color);
  }

  public int style()
  {
    return(this.style);
  }

  public String type()
  {
    return(this.type);
  }
}

// ----------------------------------------------------------------------- //

class CmdSpacing {
  int size;

  // constructor
  CmdSpacing(int size)
  {
    this.size = size;
  }

  public int size()
  {
    return(this.size);
  }
}

// ----------------------------------------------------------------------- //

class TextGroup {
  Vector types = null;
  Vector cmds = null;
  Vector measures = null;

  // constant
  final int UPPER_MARGIN = 5;
  final int SIDE_MARGIN = 5;
  final int iINIT = 0;

  int frameAlign;
  int labelAlign;
  int maxWidth = iINIT;

  // constructor
  TextGroup()
  {
    types = new Vector();
    cmds = new Vector();
    measures = new Vector();
  }


  public void add(String label, int size, Color color, int style, String type)
  {
    CommandType cType = new CommandType(CommandTypeConstant.LABEL);
    CmdLabel cmd;
    CmdMeasure measure = new CmdMeasure(label,size,style,type);
    Abc abc;
    int width;

    cmd = new CmdLabel(label, size, color, style, type);
    types.addElement(cType);
    cmds.addElement(cmd);
    measures.addElement(measure);

    // keep track of larger width
    abc = new Abc(label,size,style,type);
    width = abc.width();
    if(maxWidth==iINIT) {
      maxWidth = width;
    }
    else {
      if(maxWidth < width)
        maxWidth = width;
    }
  }

  public void add(String label, int size)
  {
    add(label, size, Color.black, Font.PLAIN, "Dialog");
  }

  public void add(int size)
  {
    CommandType cType = new CommandType(CommandTypeConstant.SPACING);
    CmdSpacing cmd;

    cmd = new CmdSpacing(size);
    types.addElement(cType);
    cmds.addElement(cmd);
  }

  public void setFrameAlign(int align)
  {
    // left(-1); center(0); right(1)
    frameAlign = align;
  }

  public void setLabelAlign(int align)
  {
    // left(-1); center(0); right(1)
    labelAlign = align;
  }

  void draw(Graphics g, int x, int y)
  {
    int n;
    int x0=x;
    int y0=y;

    switch(frameAlign) {
      case 1:  // right
               x0 -= 2*SIDE_MARGIN + maxWidth;
               break;
      case 0:  // center
               x0 -= SIDE_MARGIN + maxWidth/2;
               break;
      default: // left
    }


    n = cmds.size();
    for(int i=0; i < n; i++) {
      CommandType cType = (CommandType)types.elementAt(i);
      CmdMeasure measure = (CmdMeasure)measures.elementAt(i);
      int type = cType.type();
      switch(type) {
        case CommandTypeConstant.LABEL:
             { CmdLabel cmd;
               Abc abc;
               Point p;
               int xl=x0;

               cmd = (CmdLabel)cmds.elementAt(i);
               abc = new Abc(cmd.label(), cmd.size(), cmd.style());
               abc.setType(cmd.type());
               switch(labelAlign) {
                 case 1:  // right
                          xl += maxWidth;
                          abc.setAlign(1,1);
                          break;
                 case 0:  // center
                          xl += maxWidth/2;
                          abc.setAlign(0,1);
                          break;
                 default: // left
                          abc.setAlign(-1,1);
                          break;
               }
               p = new Point(xl,y0);
               abc.setPosition(p);
               abc.setColor(cmd.color());
               abc.draw(g);
               measure.setPosition(xl,y0);

               y0 += abc.height();
             }
             break;
        case CommandTypeConstant.SPACING:
             { CmdSpacing cmd;

               cmd = (CmdSpacing)cmds.elementAt(i);
               measure.setPosition(x0,y0);
               y0 += cmd.size(); 
             }
             break;
        default:
             System.err.println("unknown command type");
             break;
      }
    }
  }

  int width(String label)
  {
    int n = cmds.size();
    for(int i=0; i < n; i++) {
      CmdLabel cmd = (CmdLabel)cmds.elementAt(i);
      if(cmd.label.equals(label)) {
        CmdMeasure measure = (CmdMeasure)measures.elementAt(i);
        return(measure.width());
      } 
    }
    return 0;
  }

  int height(String label)
  {
    int n = cmds.size();
    for(int i=0; i < n; i++) {
      CmdLabel cmd = (CmdLabel)cmds.elementAt(i);
      if(cmd.label.equals(label)) {
        CmdMeasure measure = (CmdMeasure)measures.elementAt(i);
        return(measure.height());
      } 
    }
    return 0;
  }

}
