--------------------------------------------------------------------------------
function CampoSimples:msgHelp( configuracao )
--------------------------------------------------------------------------------
	local corpo_msg
	corpo_msg = self:formaPreenchimento()..self:restricoesLimimiteLn()..
		self:restricoesTamanhoLn()..
		self:restricoesValoresInvalidos()

	if 	configuracao == 'D' then
		return 'O campo deve ser preenchido '..corpo_msg
	elseif 	configuracao == 'AC' then
		return 'Preencha o campo '..corpo_msg
	else
		error( "Parmetro 'configuracao' invlido" ) 
	end
end

--------------------------------------------------------------------------------
function Campo:geraMensagemBKP( id_erro, parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Executa uma funo geradora de mensagem associada a um determi-
--		nado identificador de erro, passando para ela seus parmetros.
--		Para cada identificador de erro, h uma entrada na tabela 
--		'primitivas', que contm o tipo de mensagem de erro a ser gera-
--		da ('D' - diagnstico, 'C' - Ao Corretiva, ou 'DC' - 
--		Diagnstico e Ao Corretiva) e ponteiros para as funes gerado-
--		ras das mensagens dos dois tipos bsicos ('D' e 'C'). As funes
--		geradoras de mensagens esto especificadas no arquivo
--		"primitivas_semanticas.lua".
-- Parmetros:	
--		id_erro:	string representando o identificador do erro.
--		parametros: 	tabela com os parmetros da funo geradora da
--				mensagem.

-- Retorno: 	Retorna a string com a mensagem de erro gerada pela funo 
--		chamada. 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	-- Variveis
	------------------------------------------------------------------------
	local configuracao 	--auxiliar para armazenar o tipo de mensagem a  
				--ser gerada 
	local  msg		--auxiliar para armazenar a mensagem gerada
	local classe_msg 	--auxiliar para montar a tag '<P CLASS ...' com
				--estilo especificado no arquivo de  configura-
				--o do gerador (config.lua)
	
	if ESTILO_MSG then
		classe_msg = ' CLASS = "'..ESTILO_MSG..'"'
	else
		classe_msg = ""
	end
	
  	print( id_erro )
	msg = "" 
	configuracao = self.msgErro[id_erro].configuracao
	if 	configuracao == 'C' or configuracao == 'D' then
		msg = self.msgErro[id_erro][configuracao]( parametros )
	elseif configuracao == 'DC' then
		msg = self.msgErro[id_erro]['D']( parametros ).." "..
		self.msgErro[id_erro]['C']( parametros )
	end
	return '<P'..classe_msg..'>'..msg..'</P>'
end
function CampoFormatado:criticaBackup()

	caracteres_especiais_lua = { '(',')','%','.','[','*', '-', '?' }
	local erro, parametros = sel.CampoSimples:critica()
	if 	not erro then
		if 	not self.formatoLua then
			--Converte o formato especificado na sintaxe do 
			--VALIDAFORM para a da linguagem lua
			self.formatoLua = gsub(self.formato, '9', '%d')
			self.formatoLua = gsub(self.formatoLua, 'a', '%w')
			self.formatoLua = gsub(self.formatoLua, 'l', '%a')
			self.formatoLua = gsub(self.formatoLua, 'x', '%.')
			
			--Converte expresses do tipo '%%x' para x, pois
			--significam que o prprio caracter x consta do
			--formato
			self.formatoLua = gsub(self.formatoLua, '%%d', '9')
			self.formatoLua = gsub(self.formatoLua, '%%w', 'a')
			self.formatoLua = gsub(self.formatoLua, '%%a', 'l')
			self.formatoLua = gsub(self.formatoLua, '%%x', '%d')
			
			--Converte expresses do tipo <caracter especial>
			--para %<caracter_especial> pois significam que o 
			--prprio caracter <caracter_especial> consta do formato
			--os caracteres especiais so: ()%.[*-?
			
			self.formatoLua = gsub(self.formatoLua, '', '%%d')
			self.formatoLua = gsub(self.formatoLua, '%%w', 'a')
			self.formatoLua = gsub(self.formatoLua, '%%a', 'l')
			self.formatoLua = gsub(self.formatoLua, '%%x', '%d')
			
			
			
		end
		
		if 	not strfind(self.valor, self.formatoLua) then
			erro = 'tipo_invalido' 
			parametros = { tipo_campo = 'formatado', 
			formato_campo = self.formatoLua }
		end
	end
	
	return erro, parametros
end
function forma_preenchimento_campo( id_erro, parametros )
	local msg = gsub( primitivas[id_erro]['C'](parametros), "Preencha o campo", "" )
	return strsub(msg, 1,-2) 
end

function CampoComposto:preparaCoordenacaoCamposInvalidosBKP()
	--Inicializa a lista de parametros para a geracao de formas de 
	--preenchimento com o primeiro campo invlido
	local indice, parametros, incluiu
	
 	local campo, atributos = next( self.camposInvalidos, nil )
	
	if 	not atributos.parametros then
		atributos.parametros = {}
	end
	
	atributos.parametros.erro = atributos.erro
	
	local coordenadas = 
	{ { 
			id	= campo,
			campos = atributos.nome, 
			parametros = atributos.parametros } }
	
	--Para cada campo invlido, a partir do prximo, inluir seu
	--nome associado a uma forma de preenchimento na tabela. 
	--Caso sua forma seja indita,  criada uma entrada para ela 	
	
	campo, atributos = next( self.camposInvalidos, campo )
	
	while 	campo do
		--Para cada forma de preenchimento ja includa, compar-la com
		--a do campo corrente
		
		if 	not atributos.parametros then
			atributos.parametros = {}
		end
		atributos.parametros.erro = atributos.erro

		incluiu = nil
		indice = 1
		while 	coordenadas[indice] and not incluiu do
			--print( '<BR>Comparando ', atributos.nome, ' com ', 
			--coordenadas[indice].campos )
			
			if 	igual( atributos.parametros, coordenadas[indice].parametros ) then
				coordenadas[indice].campos = coordenadas[indice].campos..
				", "..atributos.nome 
				incluiu = 1
			else
				indice = indice + 1
			end
		end
		
		if 	not incluiu then
			atributos.parametros.erro = atributos.erro
			coordenadas[indice] = 
			{ 	
				id	= campo,
				campos  = atributos.nome, 
				parametros = atributos.parametros  }
			--print( '<BR>', indice, coordenadas[indice].campos )		
		end
		
		campo, atributos = next( self.camposInvalidos, campo )
	end
	
	indice = 1
	while 	coordenadas[indice] do
		--print( "<BR>", indice, 'campos: ', coordenadas[ indice ].campos )
		coordenadas[ indice ] = coordenadas[ indice ].campos.." "..
		id
		--print( '<BR>coordenadas['..indice..']: ', coordenadas[ indice ] )
		indice = indice + 1
	end	
	return coordenadas, indice-1
end

function igual( valor1, valor2 )
	local cont1, cont2 = 0, 0
	--Se os tipos so diferentes os valores so diferentes
	if 	type( valor1 ) ~= type( valor2 ) then
	 	return nil
	end
	
	--Se o valor no  estruturado (tabela), faz a comparao simples
	if 	type(valor1) ~= 'table' then
		return valor1 == valor2
	end
	
	--Se os  valores so estruturados (tabelas), s sero iguais se
	--e somente se, todos os seus campos forem iguais
	id = next( valor1, nil )
	while 	id do
		cont1 = cont1 + 1
		--print( '<BR>',id )
		if 	not igual(valor1[id], valor2[id]) then
			--print( ' diferente: valor1.'..id..' = ', 
			--valor1[id], ' e valor2.'..id..' = ', valor2[id] )
			return nil	
		end	
		--print( ' igual.<BR>' )
		id = next( valor1, id )
	end
	
	--Se houver mais campos na tabela valor2, ento elas so 
	--diferentes
	id = next( valor2, nil )
	while 	id do
		cont2 = cont2 + 1
		id = next( valor1, id )
	end
	
	if  	cont2 > cont1 then
		return nil
	end
	
	return 1
end

--------------------------------------------------------------------------------
function prepara_coordenacao_componentes_invalidos( campos_invalidos )
--------------------------------------------------------------------------------
-- Descrio: 	A partir de uma tabela contendo informaes para a gerao das 
--		mensagens de erros dos componentes de um campo composto, retorna 
--		uma tabela de strings com as submensagens a serem coordenadas. 

-- Parmetros:  campos_invalidos: Tabela que contm, associada a cada componen-
--		te invlido, uma tabela com o identificador do erro e os par-
--		metros necessrios para a gerao da mensagem de erro.

-- Retorno:	retorna uma tabela de strings com com as submensagens a serem 
--		coordenadas e o nmero de itens da tabela.

-- Autora: 	Gilda Dahis
-- Data:	07/06/1998
-- Alterado por: 
-- Em:
--------------------------------------------------------------------------------
	local itens_coordenacao, num_itens, campo, atributos, parametros
	itens_coordenacao = {}
	num_itens = 0
	campo, atributos  = next( campos_invalidos, nil )
	while 	campo do
		--print( campo )
		num_itens = num_itens + 1
		parametros = atributos.parametros
		if 	atributos.erro == 'tipo_invalido' then
			preenchimento = forma_preenchimento( 
						parametros.tipo_campo, parametros.formato_campo )
		elseif atributos.erro == 'limite_ultrapassado' then
			preenchimento = forma_preenchimento( 
						parametros.tipo_campo ).." "..
  			descricao_restricao( parametros.restricao, 
  						parametros.tipo_campo, 
  						parametros.valor_restricao )
		--elseif atributos.erro == restricao_tamanho_insatisfeita =
		end
		
		itens_coordenacao[ num_itens ] = 'o campo '..atributos.nome..
		' '..preenchimento
		--print( itens_coordenacao[num_itens], '<BR>' )
		campo, atributos  = next( campos_invalidos, campo )
	end
	return itens_coordenacao, num_itens
end
--------------------------------------------------------------------------------
function CampoEstruturado:msgCorrigirComponentes()
--------------------------------------------------------------------------------
-- Descrio: 	Primitiva geradora da mensagem relacionada ao erro de preenchi-
--		mento incorreto de componentes de um campo estruturado.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local genero_componente = genero(self.nomeComponente)

	------------------------------------------------------------------------
	if 	self.tipoMsg == 'global_local'  then
	------------------------------------------------------------------------
		local itens_coordenacao, num_itens = 
  		prepara_coordenacao_componentes_invalidos( self.camposInvalidos, 
  		self.nomeComponente ) 

		return "Corrija "..
		unifica_numero(genero_componente, num_invalidos).." "..
		unifica_numero(self.nomeComponente, num_itens).." "..
		coordena( itens_coordenacao, num_itens, 'e' )..
		", seguindo as instrues junto "..
		unfica_numero(conjuncao( 'a', genero_componente)).." "..
		unifica_numero( unifica_genero('mesmo', genero_componente ) ).."."
	end
	
	------------------------------------------------------------------------
	--self.tipoMsg == 'global'  
	------------------------------------------------------------------------
  	local itens_coordenacao, num_itens, campo_mais_detalhado = 
  	prepara_coordenacao_componentes_invalidos( self.camposInvalidos, 
  						self.nomeComponente ) 
  	local 	pronome_obliquo
  	if 	num_itens > 1 then
  		pronome_obliquo = genero_componente.."s"	
  	else
  		pronome_obliquo =  genero_componente
  	end							
  	
  	local 	preenchimento = forma_preenchimento_elemento( 
  	self.camposInvalidos[campo_mais_detalhado] )	
  	return "Corrija "..coordena( itens_coordenacao, num_itens, 'e' )..' '..
  		self.nomeComponente.." preenchendo-"..pronome_obliquo.." "..
  		preenchimento.."."		
end

--------------------------------------------------------------------------------
function CampoSimples:msgSelecionarNaLista()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		'chave' com um valor j cadastrado no BD. Esta primitiva deve ser 
--		usada em casos onde h uma interface que disponibiliza a seleo 
--		de um campo chave existente em uma lista ou a criao de uma nova 
--		chave. 
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return 'Selecione "'..self.valor..'" na lista de '..
  	plural( primeira_maiuscula(self.nome ) )..'.'
end
--------------------------------------------------------------------------------
function restricaoLimiteLn( operador )
--------------------------------------------------------------------------------
-- Descrio: 	Dada um operador de uma restrio de limite a ser satisfeita 
--		por um campo, da forma <valor do campo> <operador> <operando>, 
--		exemplo:
--		
--		total de avaliaes <= 15 
--	
--		descreve o operador e o operando da direita em linguagem 
--		natural. 
--
-- Parmetros:	operador: string indicando o operador de comparao da 
--		restrio (pode ser '<', '<=', '>' ou '>='.
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve uma restrio de limite. 
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  if 	not operador then
	error("O parmetro 'operador'  nil.")
  end

  --Identifica o operando da restrio a ser utilizado para a descrio em lin-
  --guagem natrual: pode ser um valor ou um descritor de um valor
  local operando = self:operandoLn( operador )
  
  local genero_descritor
  -------------------------------------------------------------------
  --Se o descritor do valor da restrio  um nmero, o gnero  nulo
  -------------------------------------------------------------------
  if 	strfind( operando, "^%d%d*$" ) or 
	strfind( operando, "^%d%d*%.%d%d*$" ) then
 	genero_descritor = ""
  else
  -----------------------------------------------------------------------------	
  --Caso contrrio, o gnero  recuperado atravs do dicionrio. Se o descritor 
  --do valor  uma locuo, o gnero da locuo  o genro do ncleo. 
  --Para simplificar, pegaremos o gnero da primeira palavra da locuo
  ------------------------------------------------------------------------------	
  	inicio = strfind( operando, " " )
	
	if 	inicio then 
		genero_descritor = genero(strsub(operando,1,inicio-1))
	else
 		genero_descritor = genero(operando)
	end  
  end
  
  return self:operadorLn( operador, genero_descritor ).." "..operando
end
--------------------------------------------------------------------------------
function CampoComposto:preparaCoordenacaoCamposInvalidos_bkp()
--------------------------------------------------------------------------------
-- Descrio: 	A partir do membro self.camposInvalidos, que contm informaes
--		para a gerao das mensagens de erros dos componentes que foram 
--		preenchidos de forma invlida, cria uma tabela de submensagens
--		que agrupam os campos que possuem mensagens de erro idnticas. 
--		Exemplo:
--		Dadas as seguintes mensagens de componentes:

--		msg do campo data: 'com uma data no formato "dd/mm/aaaa"'
--		msg do campo hora: 'com um valor maior do que 0'
--		msg do campo minuto: 'com um valor maior do que 0'

--		A tabela criada seria:

--		{ 	"o campo data com uma data no formato "dd/mm/aaaa",
--			"os campos hora e minuto com um valor maior do que 0"
--		}

-- Retorno:	retorna uma tabela de strings contendo submensagens que agrupam 
--		os campos que possuem mensagens de erro idnticas e o nmero de 
--		itens da tabela.

-- Autora: 	Gilda Dahis
-- Data:	07/06/1998
-- Alterado por: 
--------------------------------------------------------------------------------
	--Inicializa a lista de parametros para a geracao de formas de 
	--preenchimento com o primeiro campo invlido
	local indice, parametros, incluiu
	
 	local campo, atributos = next( self.camposInvalidos, nil )
	local campo_aux
	
	--if 	not atributos.parametros then
	--	atributos.parametros = {}
	--end
	
	--atributos.parametros.erro = atributos.erro
	local coordenadas = 
	{ { 
			id	= campo,
			campos = atributos.nome,
			numCampos = 1, 
			erro   = atributos.erro,
			parametros = atributos.parametros 
	} }
	
	--Para cada campo invlido, a partir do prximo, inluir seu
	--nome associado a uma forma de preenchimento na tabela. 
	--Caso sua forma seja indita,  criada uma entrada para ela 	
	
	campo, atributos = next( self.camposInvalidos, campo )
	
	while 	campo do
		--Para cada forma de preenchimento ja includa, compar-la com
		--a do campo corrente
		
		--if 	not atributos.parametros then
		--	atributos.parametros = {}
		--end
		--atributos.parametros.erro = atributos.erro
		incluiu = nil
		indice = 1
		while 	coordenadas[indice] and not incluiu do
			--print( '<BR>Comparando ', atributos.nome, ' com ', 
			--coordenadas[indice].campos )
			
			campo_aux = coordenadas[indice]
			if 	self.componentes[campo_aux.id]:mesmaMsg(campo_aux, 
				self.componentes[campo], atributos) then
				coordenadas[indice].campos = campo_aux.campos..
				", "..atributos.nome 
				coordenadas[indice].numCampos = coordenadas[indice].numCampos+1
				incluiu = 1
			else
				indice = indice + 1
			end
		end
		
		if 	not incluiu then
			--atributos.parametros.erro = atributos.erro
			coordenadas[indice] = 
			{ 	
				id	= campo,
				campos  = atributos.nome, 
				numCampos = 1,
				erro 	= atributos.erro,
				parametros = atributos.parametros  }
			--print( '<BR>', indice, coordenadas[indice].campos )		
		end
		
		campo, atributos = next( self.camposInvalidos, campo )
	end
	local msg_tipo
	indice = 1
	while 	coordenadas[indice] do
		--print( "<BR>", indice, 'campos: ', coordenadas[ indice ].campos )
		campo_aux = coordenadas[ indice ]
		msg_tipo = gsub(componentes[campo_aux.id]:msgErro( campo_aux.erro, campo_aux.parametros),
		"Preencha o campo ", "preenchendo-"..unifica_numero('o', campo_aux.numCampos).." " )
		msg_tipo = strsub( msg_tipo, 1,-2)
		coordenadas[ indice ] = self:formataCoordenacao(coordenadas[ indice ].campos)..
		" "..msg_tipo
		--print( '<BR>coordenadas['..indice..']: ', coordenadas[ indice ] )
		indice = indice + 1
	end	
	return coordenadas, indice-1
end
--------------------------------------------------------------------------------
function CampoVetor:geraMensagensLocais()
--------------------------------------------------------------------------------
-- Descrio:	Gera as mensagens associadas aos componentes invlidos de um 
--		campo vetor, a partir das informaes armazenadas na 
--		tabela membro camposInvalidos.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local id, atributos = next( self.camposInvalidos, nil )
	while 	id do
		cgi['msg_'..id] = 
		self.componentes:geraMensagem( atributos.erro, atributos.parametros )
		--print('cgi.msg_'..id, "=", cgi['msg_'..id], '<BR>')
		id, atributos = next( self.camposInvalidos, id )
	end
end

function CampoComposto:guardaIdComponentes()
	local ids = {}
	local componentes = self.componentes
	local i = 1
	while 	componentes[i] do
		ids[i] = componentes[i].identificador
		i = i + 1
	end
	return ids
end

function CampoVetor:guardaIdComponentes()
	ids = self.componentes:guardaIdComponentes()
	return ids
end
