----------------------------- campo.lua ---------------------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	campo.lua
-- Descrio:  	Contm a definio e os mtodos da classe abstrata Campo. Um 
--		objeto desta classe representa um campo de entrada de dados de 
--		um formulrio html, contendo informaes semnticas sobre ele. 
--		Os mtodos especficos das subclasses da classe Campo permitem 
--		a crtica dos valores digitados em diferentes tipos de campos.
-- Data:	21/04/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes: 
--     Quem      |   Quando   |   Motivo
-- --------------|------------|-------------------------------------------------
--  Gilda	  07/08/1999   Incluso do parmetro identificador como segundo
--				parmetro do construtor.
--------------------------------------------------------------------------------

--Quando estiver debugando, retire os comentrios das 2 linhas a seguir
--$debug
--cgilua.htmlheader()

--------------------------------------------------------------------------------
--Carrega funes de outros mdulos
--------------------------------------------------------------------------------

--Biblioteca com as funes qur permitem a implementao orientada a objetos,
--contendo a definio da classe bsica Object.
dofile('object.lua')

--------------------------------------------------------------------------------
-- Definio da Classe Campo, como uma subcalsse da classe Object
--------------------------------------------------------------------------------
-- Um objeto desta classe representa um campos de um formulrio html, 
-- contendo informaes semnticas sobre ele e o valor digitado dinmicamente 
-- pelo usurio.

--print( 'Criao da classe Campo<BR>' )
Campo = Object:subclass {
	numCampos	= 3,
	nome=nil,	--string indicando o nome ou apelido, pelo qual o usu-
			--rio identifica o campo na interface 
	identificador   =nil,
	obrigatorio=nil--este atributo serve para indicar se o preenchimento do
			--campo  o obrigatrio ou no. Quando for diferente de
			--nil o preenchimento  obrigatrio, em caso contrrio
			--no . 
}

--------------------------------------------------------------------------------
-- Definio dos mtodos da Classe Campo
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Construtor
--------------------------------------------------------------------------------
function Campo:new (o)
--------------------------------------------------------------------------------
-- Descrio: Construtor dos objetos da classe Campo.

-- Parmetros:	o: tabela com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	

-- Retorno: 	Retorna o objeto da classe Campo criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --print('Entrando em Campo:new()<BR>' )	
   local p, ultimo_indice --Auxiliares para a criao do objeto
   
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.parent 	= self
   
   --Descobre o ltimo ndice dos membros da superclasse
   ultimo_indice = self.parent.ultimoIndice

   --Atribui valores aos campos especficos da classe
   p.nome 		= o[ultimo_indice+1]
   p.identificador 	= self:inicializaMembro(o, ultimo_indice+2, 'string', 'Campo')
   p.obrigatorio	= o[ultimo_indice+3]
   p.tipo 		= 'campo'

   --print('Saindo de Campo:new()<BR>' )	
   --p:print()
   return p
end

--------------------------------------------------------------------------------
function Campo:identifica( prefixo_identificador, i )
--------------------------------------------------------------------------------
-- Descrio: 	Concatena ao identificador de um campo elemento de um vetor, ou
--		subcampo do campo elemento (quando este  estruturado) o ndice 
--		corrente do elemento sendo analisado. Pois, conforme explicado 
--		na descrio do parmetro id_elemento da funo 
--		critica_campo_vetor(), no formulrio html, as tags html que ge-
--		ram a exibio dos campos do vetor, possuem o atributo NAME no 
--		seguinte formato <id_elemento><i>. Mas a especificao dos 
--		atributos do campo elemento no arquivo do tipo "form*.lua" 
--		indica apenas o prefixo <id_elemento>, pois a semntica do cam
--		po  a mesma para todos os elementos, bastando ser indicada uma
--		vez. Alm do que, o nmero de elementos preenchidos  determina-
--		do dinamicamente.  necessrio, ento, calcular o identificador 
--		de cada elemento e de seus subcampos, dinamicamente, para 
--		determinar o nome da varivel cgi que contm o valor dinmico do 
--		elemento (ou subcampo), da forma cgi.<identificador><i> e tambm 
--		qual ser o nome da varivel que armazenar a mensagem de erro 
--		associada ao elemento (ou subcampo), da forma 
--		cgi.msg_<identificador><i>.

-- Parmetros:	
--		i: inteiro representando o ndice do elemento corrente sendo 
--		criticado.

-- Retorno:	Esta funcao no retorna nada. Sua sada est representada pela 
--		atribuio dos valores calculados: o identificador e o valor do
--		elemento ou subcampo de ndice corrente na tabela campos, res-
--		pectivamente, nos membros:

--		self.identificador:	string que armazenar o identificador do
--					 elemento ou subcampo corrente.
--		self.valor:		Membro que armazenar o valor dinmico do 
--					elemento ou subcampo.

-- Autora: 	Gilda Dahis
-- Data: 	05/11/1998
--------------------------------------------------------------------------------
	self.identificador = prefixo_identificador..i
	self.valor	= cgi[self.identificador]
	if 	self.valor then
		self.tamanhoValor = strlen(self.valor)
	else
		self.tamanhoValor = 0	
	end
	--print( self.identificador, " = ", self.valor, '<BR>' )
end

function Campo:prefixoIdentificador()
	return self.identificador
end
