----------------------------- debug.lua ---------------------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	debug.lua
-- Descrio:  	Contm funes para auxlio  debugao e testagem da 
--		biblioteca. Estas funes executam funes da biblioteca geran-
--		do tags html para a visualizao de resultados da execuo. 
--		Estas tags podem ser utilizadas tanto para a gerao de pginas 
--		dinmicas como para a gerao de arquivos em disco.

--		
-- Data:	23/07/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes: 
--     Quem      |   Quando   |   Motivo
-- --------------|------------|-------------------------------------------------
--  Gilda	  07/08/1999   Mudana na estrutura do membro campos da classe
--				Formulario, agora  indexado por nmeros.
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--Funes auxiliares utilizadas pelos mtodos de teste.
--------------------------------------------------------------------------------
function formata_msg( msg )
--------------------------------------------------------------------------------
-- Descrio: 	transforma as tags html geradas para a exibio de uma mensagem
--		de erro, substituindo o estilo especificado atravs da varivel 
--		global de configurao 'ESTILO_MSG' para um estilo de fonte na
--		cor vermelha, em itlico e sem pargrafo. 
-- Parmetros:	msg: string com as tags html a serem transformadas.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	local classe_msg
	if ESTILO_MSG then
		classe_msg = ' CLASS = "'..ESTILO_MSG..'"'
	else
		classe_msg = ""
	end
	msg = tostring( msg )
	msg = gsub(msg, '<P'..classe_msg..'>', '<FONT color = "red"><I>')
	msg = gsub(msg, '</P>', '</I></FONT>')	
	return msg
end


--------------------------------------------------------------------------------
-- Classe Campo
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function Campo:atribuiValor( valor )
--------------------------------------------------------------------------------
-- Descrio: 	Atribui um valor ao membro valor do campo. (Para testar a
--		crtica do campo, quando ele  preenchido com este valor).
-- Parmetros:	valor: valor a ser testado, pode ser uma string ou nil.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	self.valor = valor
	if 	valor then
		self.tamanhoValor = strlen( valor )
	end
end

--------------------------------------------------------------------------------
function Campo:imprime()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para 
--		a exibio dos atributos de um objeto da classe Campo.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	write('<B>Nome:</B> '..self.nome..'<BR>\n')
	write('<B>Tipo:</B> '..self.tipo..'<BR>\n')
end

--------------------------------------------------------------------------------
function Campo:testa( valores )
--------------------------------------------------------------------------------
-- Descrio: 	Testa a crtica e a gerao de mensagens para o campo. O teste
--		 feito a partir de um conjunto de valores simulados. Gera, 
--		para a sada padro (disco ou browser), tags html para exibio 
--		de uma tabela que relaciona cada valor testado com as mensagens 
--		de erro geradas, nas duas configuraes possveis (C-Corretiva, 
--		D-Diagnstico).
-- Parmetros:	valores: tabela com os valores simulados para o teste.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	--Imprime o nome do campo
	write('<B>Campo '..self.nome..'</B><BR>\n')
	write('<TABLE BORDER = "1" CELLSPACING = "0">\n')
	write('<TR>\n<TD ALIGN = "center"><B>Valor</B></TD>\n<TD ALIGN = "center"><B>Conf.</B></TD>\n<TD ALIGN = "center"><B>Mensagem</B></TD>\n</TR>\n')
	--Para cada valor 
	local i = 1
	while valores[i] do	
		--testar
		self:testaValor( valores[i] )
		i = i + 1
	end
	write("</TABLE>\n")
end

--------------------------------------------------------------------------------
function Campo:testaGeracaoMsg( id_erro, parametros, configuracao )
--------------------------------------------------------------------------------
-- Descrio:	Testa a gerao de mensagem associada a um determinado erro e em 
--		uma determinada configurao, retornando a mensagem gerada.
-- Parmetros:	id_erro: string que indica o erro a ser testado.
--		parametros: tabela com os parmetros requeridos pela funo ge-
--		radora de mensagem associada ao erro.

--		configuracao: caracter que indica a configuracao a ser testada.

-- Retorno:	retorna para a sada padro (browser ou disco) a mensagem 
--		gerada no teste, formatada com fonte na cor vermelha e em 
--		itlico.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	--print( id_erro, configuracao )
	if 	id_erro then
		self.parent.configuracaoMsgErro[id_erro] = configuracao
		return '<FONT color = "red"><I>'..
		tostring(self:msgErro(id_erro, parametros))..'</I></FONT>'
	else
		return '&nbsp;'	
	end
end

--------------------------------------------------------------------------------
function Campo:testaValor( valor )
--------------------------------------------------------------------------------
-- Descrio:	Testa a crtica e a gerao de mensagem do campo, simulando o
--		seu preenchimento com um determinado valor. Gera, para a sada 
--		padro (disco ou browser), tags html para exibio das mensagens 
--		de erro, caso tenham sido geradas, nas duas configuraes 
--		possveis (C-Corretiva, D-Diagnstico).

-- Parmetros:	valor: valor a ser testado, pode ser uma string ou nil.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	self:atribuiValor( valor )
	local id_erro, parametros = self:critica()
	local valor

	valor = tostring(self.valor) 
	if 	valor == '' then
		valor = '&nbsp;'
	end

	write('<TR>\n<TD ALIGN = "center" ROWSPAN = "2">'..valor..
	'</TD>\n')
	write('<TD ALIGN = "center">C</TD>\n')		
	write('<TD>'..
	self:testaGeracaoMsg(id_erro, parametros, 'C')
	..'</TD>\n</TR>\n')
	write('<TR><TD ALIGN = "center">D</TD>\n')		
	write('<TD>'..
	self:testaGeracaoMsg(id_erro, parametros, 'D')
	..'</TD>\n</TR>')
end


--------------------------------------------------------------------------------
-- Classe CampoIlimitado
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoIlimitado:imprimeRestricoesTamanho()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para a
--		exibio das restries de tamanho do campo.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	write('<B>Restries de Tamanho:</B>\n<UL>')
	local tabela = self.restricoes.tamanho
	local operador, operando  = next( tabela, nil )
	while 	operador do
		write('\n<LI>'..operador..' '..operando)
		operador, operando = next( tabela, operador )
	end
	write('\n</UL>\n')
end

--------------------------------------------------------------------------------
-- Classe CampoSimples
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoSimples:imprime()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para 
--		a exibio dos atributos especficos de um objeto da classe 
--		CampoSimples.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	--Gatilho para chamar uma funo da superclasse com o mesmo nome
	local aux = CampoSimples.imprime
	CampoSimples.imprime = nil
	self:imprime()
	CampoSimples.imprime = aux

	--Se o campo tem restries
	if 	self.restricoes then
		--imprime os atributos das restries
		if 	self.restricoes.limite then
			self:imprimeRestricoesLimite()			
		end

		if 	self.restricoes.tamanho then
			self:imprimeRestricoesTamanho()		
		end

		if 	self.restricoes.valores_invalidos then
			write('<B>Valores Invlidos:</B> '..
			self.restricoes.valores_invalidos..'<BR>\n')
		end
	end

	--Se o campo  do tipo chave
	if 	self.atributosChave then
		--imprime os atributos chave
		write('<B>Atributos de campo chave:</B>\n')
		write('<UL>\n')
		write('<LI><B>tabela:</B> '..self.atributosChave.tabela..'<BR>\n')
		write('<LI><B>campo:</B> '..self.atributosChave.campo..'<BR>\n')
		write('<LI><B>tipoBD:</B> '..tostring(self.atributosChave.tipoBD)..'<BR>\n')
		write('</UL>\n')
	end	
end

--------------------------------------------------------------------------------
function CampoSimples:imprimeRestricoesLimite()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para a
--		exibio das restries de limite do campo.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	write('<B>Restries de Limite:</B>\n<UL>')
	local tabela = self.restricoes.limite
	local operador, operando  = next( tabela, nil )
	while 	operador do
		write('\n<LI>'..operador..' '..operando.valor)
		if 	operando.nome then
			write(' ('..operando.nome..')')
		end
		operador, operando = next( tabela, operador )
	end
	write('\n</UL>\n')
end

--------------------------------------------------------------------------------
-- Classe CampoFormatado
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoFormatado:imprime()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para 
--		a exibio dos atributos especficos de um objeto da classe 
--		CampoFormatado.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	--Gatilho para chamar uma funo da superclasse com o mesmo nome
	local aux = CampoFormatado.imprime
	CampoFormatado.imprime = nil
	self:imprime()
	CampoFormatado.imprime = aux
	write('<B>Formato:</B> '..self.formato..'<BR>\n' )
end

--------------------------------------------------------------------------------
-- Classe CampoData
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoData:imprime()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para 
--		a exibio dos atributos especficos de um objeto da classe 
--		CampoData.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	--Gatilho para chamar uma funo da superclasse com o mesmo nome
	local aux = CampoData.imprime
	CampoData.imprime = nil
	self:imprime()
	CampoData.imprime = aux

	write('<B>Interface:</B> '..self.interface..'<BR>\n' )
end

--------------------------------------------------------------------------------
-- Classe CampoEstruturado
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoEstruturado:imprime()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para 
--		a exibio dos atributos especficos de um objeto da classe 
--		CampoEstruturado.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	--Gatilho para chamar uma funo da superclasse com o mesmo nome
	local aux = CampoEstruturado.imprime
	CampoEstruturado.imprime = nil
	self:imprime()
	CampoEstruturado.imprime = aux
	self:imprimeComponentes()
end

--------------------------------------------------------------------------------
function CampoEstruturado:testa( testes, prefixo_descricao )
--------------------------------------------------------------------------------
-- Descrio:	Executa um conjunto de testes do campo estruturado, gerando para 
--		a sada padro (browser ou disco) tags html para a exibio das 
--		condies e resultados do teste. 
-- Parmetros:	testes: tabela com os casos de teste. Esta tabela possui os se-
--		guintes ndices:
--		descricao: string contendo uma descrio para o caso de teste, 
--		pode ser nil. Neste caso no  gerada tag para a exibio da 
--		descrio.	
--		restricoes: tabela contendo restricoes a serem testadas. Tem a
--		mesma estrutura que o membro restricoes do campo.
--		cgi: tabela contendo uma tabela com os valores simulados para 
--		cada componente do campo. Cada valor  indexado pela string que
--		identifica o componente (seu membro identificador).

--		prefixo_descricao: string a ser exibida como prefixo da	des-
--		crio do teste. Por exemplo se se deseja gerar a exibio de
--		um conjunto de testes denominados "Teste i", passaramos 
--		"Teste: " neste parmetro. Se na verdade estamos gerando a exi-
--		bio de casos de testes, poderamos passar "Caso 1.". 
--		As descries geradas incializariam-se com "Caso 1.1", 
--		"Caso 1.2", etc.

-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	local i = 1
	while( testes[i] ) do
		--Inicializa os parametros do teste
		self.restricoes = testes[i].restricoes
		self:atribuiDefaultRestricoes()
		self:atribuiValor( testes[i].cgi )	
		write('<HR size = "1"><B>'..prefixo_descricao..i)
		if 	testes[i].descricao then 
			write(': '..testes[i].descricao)
		end
		write('</B><HR size = "1">')
		self:testaRestricoesSubconjunto()
		i = i + 1
	end
end

--------------------------------------------------------------------------------
function CampoEstruturado:testaCaso( testes, obrigatorio )
--------------------------------------------------------------------------------
-- Descrio:	Executa um conjunto de testes do campo estruturado, simulando um 
--		determinado valor do membro 'obrigatorio', gerando para a sada 
--		padro (browser ou disco) tags html para a exibio das 
--		condies e resultados do teste. 
-- Parmetros:	testes: tabela com os casos de teste. Esta tabela possui os se-
--		guintes ndices:
--		descricao: string contendo uma descrio para o caso de teste, 
--		pode ser nil. Neste caso no  gerada tag para a exibio da 
--		descrio.	
--		restricoes: tabela contendo restricoes a serem testadas. Tem a
--		mesma estrutura que o membro restricoes do campo.
--		cgi: tabela contendo uma tabela com os valores simulados para 
--		cada componente do campo. Cada valor  indexado pela string que
--		identifica o componente (seu membro identificador).

--		obrigatorio: valor do membro obrigatrio, para o teste.		
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	local descricao, caso 
	self.obrigatorio = obrigatorio
	if 	obrigatorio then
		descricao = 'Caso 2: o campo  obrigatrio'
		caso = 2
	else
		descricao = 'Caso 1: o campo no  obrigatrio'
		caso = 1
	end
	write('<HR size = "1"><B>'..descricao..'</B>')
	self:testa( testes, "Caso "..caso.."." )
end

--------------------------------------------------------------------------------
function CampoEstruturado:testaParametros( configuracaoMsg, tipo_msg )
--------------------------------------------------------------------------------
-- Descrio:	Testa a crtica e a gerao de mensagens do campo para uma de-
--		terminada configurao de mensagem (Corretiva ou de 
--		Diagnstico) e para uma determinada poltica de	gerao de men-
--		sagens para campo estruturado (somente mensagem global, mensagem 
--		global e mensagens locais, somente mensagem local), gerando para 
--		a sada padro (browser ou disco) tags html para a exibio das 
--		mensagens geradas.
-- Parametros:	configuracaoMsg: caracter que indica a configurao de estilo de
--		mensagem a ser testada ('C' para Ao Corretiva e 'D' para 
--		Diagnstico).
--		tipo_msg: string que indica a configurao para a gerao de 
--		mensagens relacionadas ao campo estruturado a ser utilizada no 
--		teste. Pode ser 'global', 'global_local' ou 'local'.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	--Reinicializa os contadores e tabelas de campos vazios 
	self.contVazios = 0
	self.camposVazios = {}
	self.contInvalidos = 0
	self.camposInvalidos = {}
	
	self.tipoMsg = tipo_msg
	cgi.configuracao = configuracaoMsg
	chdir(cgilua.script_pdir..CAMINHOS[cgilua.script_vdir])
	dofile('configura_msgs.lua')
	chdir(cgilua.script_pdir)
	--print( 'fax_interno: -', self.componentes.fax_interno.valor, '-' )
	
	--print( 'cgi.configuracao', cgi.configuracao, '<BR>' )
	id_erro, parametros = self:critica()
	write('<TR>\n<TD ALIGN = "center">'..configuracaoMsg..'</TD>\n')
	write('<TD ALIGN = "center">'..self.tipoMsg..'</TD>\n')
	write('<TD>')
	--print( id_erro )
	write(self:testaGeracaoMsg( id_erro, parametros, configuracaoMsg)) 
	write('</TD>\n')
	
	--Imprime as mensagens locais geradas
	write('<TD ALIGN = "left">')
	self:testaGeracaoMsgLocais()
	write('</TD>')
end

--------------------------------------------------------------------------------
function CampoEstruturado:testaRestricoesSubconjunto()
--------------------------------------------------------------------------------
-- Descrio:	Testa a crtica do campo gerando para a sada padro (browser ou 
--		disco) tags html para a exibio de suas restries, dos valores 
--		simulados para os componentes e das mensagens de erro geradas, 
--		de todas as formas possveis: somente mensagem global, mensagem 
--		global e local e somente mensagem local, nas duas configuraes 
--		possveis: mensagens corretivas e mensagens de diagnstico.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	if 	self.restricoes then
		write('<B>subconjunto:</B> "'..self.restricoes.subconjunto..'"<BR>\n')
		self:imprimeRestricoesTamanho()		
	end

	--Imprime os valores do teste
	self:imprimeValoresComponentes()
	--Para cada combinao de parmetros tipoMsg e configuracaoMsg,
	--testa a execuo da funo de crtica
	write('<BR>\n<B>Mensagens Geradas:</B><BR>')

	write('<TABLE BORDER = "1" CELLSPACING = "0">\n')
	write('<TR>\n<TD ALIGN = "center"><B>conf.</B></TD>\n<TD ALIGN = "center"><B>tipoMsg</B></TD>\n')
	write('<TD ALIGN = "center"><B>Msg Global</B></TD><TD ALIGN = "center"><B>Msgs Locais</B></TD></TR>\n')
		
	self:testaParametros( 'C', 'global')
	self:testaParametros( 'C', 'global_local')
	self:testaParametros( 'C', 'local')

	self:testaParametros( 'D', 'global')
	self:testaParametros( 'D', 'global_local')
	self:testaParametros( 'D', 'local')
	write('</TABLE>')
end

--------------------------------------------------------------------------------
-- Classe CampoComposto
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoComposto:atribuiValor( valores )
--------------------------------------------------------------------------------
-- Descrio: 	Atribui valores determinados aos componentes do campo composto. 
--		(Para testar a crtica do campo, quando os componentes so pre-
--		enchidos  com estes valores).
-- Parmetros:	valores: tabela com os valores a serem atribuidos aos compo-
--		nentes. Cada valor  indexado pela string que identifica o 
--		componente (seu membro identificador).
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
   	local componentes, i
	componentes = self.componentes
	i = 1
	while( componentes[i] ) do
		--print( componentes[i].identificador, "-", valores[nome_campo], 
		--"-", '<BR>' )		
		componentes[i]:atribuiValor(valores[componentes[i].identificador])
		i = i + 1
	end
end

--------------------------------------------------------------------------------
function CampoComposto:imprimeComponentes()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para a
--		exibio dos atributos dos componentes do campo.
-- Autora: 	Gilda Dahis
-- Data: 	24/07/1999
--------------------------------------------------------------------------------
	write('<HR size = "1"><B>Componentes:</B><HR size = "1">')
   	local componentes, i
	i = 1
	componentes = self.componentes
	while( componentes[i] ) do
		componentes[i]:imprime()
		write('<HR size = "1">')
		i =  i + 1
	end
end

--------------------------------------------------------------------------------
function CampoComposto:imprimeValoresComponentes()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para a
--		exibio de uma tabela relacionando os nomes dos componentes 
--		do campo com seus valores.
-- Autora: 	Gilda Dahis
-- Data: 	24/07/1999
--------------------------------------------------------------------------------
	local valor
	write('<B>Valores dos componentes:</B>\n')
	write('<TABLE BORDER = "1" CELLSPACING = "0">\n')
	write('<TR><TD ALIGN = "center"><B>campo</B></TD><TD ALIGN = "center"><B>Valor</B></TD></TR>')
 	local componentes, i
	componentes = self.componentes
	i = 1
	while( componentes[i] ) do
		write('<TR>\n<TD ALIGN = "center">'..componentes[i].nome..'</TD>\n')
		valor = tostring( componentes[i].valor )
		if 	valor == '' then
			valor = "&nbsp;"
		end
		write('<TD ALIGN = "center">'..valor..'</TD>\n</TR>')
		i = i + 1
	end
	write('</TABLE>\n')
end

--------------------------------------------------------------------------------
function CampoComposto:testaGeracaoMsgLocais()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para a
--		exibio das mensagens locais geradas para os componentes do 
--		campo durante o teste corrente.
-- Autora: 	Gilda Dahis
-- Data: 	24/07/1999
--------------------------------------------------------------------------------
	--Para cada componente
	--Imprime o nome a mensagem gerada, se houver
	local componentes, i, id_campo
	componentes = self.componentes
	i = 1
	while( componentes[i] ) do
		id_campo = componentes[i].identificador
		write( '<B>'..componentes[i].nome..":</B> ")
		--write('<FONT color = "red"><I>'..
		--tostring(cgi['msg_'..id_campo])..'</I></FONT><BR>\n')
		write(formata_msg( cgi['msg_'..id_campo] ), '<BR>' )
		--Apaga a mensagem para a preparao do prximo teste
		cgi['msg_'..id_campo] = nil
		i = i + 1
	end
end


--------------------------------------------------------------------------------
-- Classe CampoVetor
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoVetor:atribuiValor( valores )
--------------------------------------------------------------------------------
-- Descrio: 	Atribui valores determinados aos elementos do campo vetor. 
--		(Para testar a crtica do campo, quando os componentes so pre-
--		enchidos  com estes valores).
-- Parmetros:	valores: tabela com os valores a serem atribuidos aos compo-
--		nentes. Cada valor  indexado pela string que identifica o 
--		elemento. Esta string  da forma <id_elemento><i>, onde 
--		id_elemento  o membro id_elemento do campo vetor e i  o ndice
--		do elemento.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	cgi = valores
end

--------------------------------------------------------------------------------
function CampoVetor:imprimeComponentes()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para a
--		exibio dos atributos dos componentes do campo.
-- Autora: 	Gilda Dahis
-- Data: 	24/07/1999
--------------------------------------------------------------------------------
	write('<B>Nmero de elementos:</B> '..self.numComponentes)
	write('<HR size = "1"><B>Atributos do campo Elemento:</B><HR size = "1">')
	self.componentes:imprime()
end

--------------------------------------------------------------------------------
function CampoVetor:imprimeValoresComponentes()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para a
--		exibio de uma tabela relacionando os nomes dos componentes 
--		do campo com seus valores.
-- Autora: 	Gilda Dahis
-- Data: 	24/07/1999
--------------------------------------------------------------------------------
	write('<B>Valores dos elementos:</B>\n')
	write('<TABLE BORDER = "1" CELLSPACING = "0">\n')
	write('<TR><TD ALIGN = "center"><B>elemento</B></TD><TD ALIGN = "center"><B>Valor</B></TD></TR>')
 	local i = 1
	while( i <= self.numComponentes ) do
		self.componentes:identifica( self.prefixosComponentes, i )
		write('<TR>\n<TD ALIGN = "center">'..self.componentes.identificador..'</TD>\n')
		write('<TD ALIGN = "center">'..self.componentes.valor..'</TD>\n</TR>')
		i = i + 1
	end
	write('</TABLE>\n')
end

--------------------------------------------------------------------------------
function CampoVetor:testaGeracaoMsgLocais()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para a
--		exibio das mensagens locais geradas para os componentes do 
--		campo durante o teste corrente.
-- Autora: 	Gilda Dahis
-- Data: 	24/07/1999
--------------------------------------------------------------------------------
	--Para cada componente
	--Imprime o nome a mensagem gerada, se houver
	local idElemento = self.prefixosComponentes
	if 	type(idElemento) == table then
		idElemento = idElemento.elemento
	end 
	local i = 1
	while( i <= self.numComponentes ) do
		write( idElemento..i..": ")
		write('<FONT color = "red"><I>'..
		tostring(cgi['msg_'..idElemento..i])..'</I></FONT><BR>\n')
		--Apaga a mensagem para a preparao do prximo teste
		cgi['msg_'..idElemento..i] = nil
		i = i + 1
	end
end

--------------------------------------------------------------------------------
-- Classe Formulario
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function Formulario:atribuiValores( valores )
--------------------------------------------------------------------------------
-- Descrio: 	Atribui valores determinados aos campos do formulrio. 
--		(Para testar a crtica, quando os campos so preenchidos  com 
--		estes valores).
-- Parmetros:	valores: tabela com os valores a serem atribuidos aos campos.
--		Cada valor  indexado pela string que identifica o campo 
--		(seu membro identificador).
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
   	local i = 1
	local campos = self.campos
	while( campos[i] ) do
		campos[i]:atribuiValor( valores[campos[i].identificador] ) 
		i = i + 1
	end
end

--------------------------------------------------------------------------------
function Formulario:guardaMsgs()
--------------------------------------------------------------------------------
-- Descrio: 	Depois que so executados testes da gerao de mensagem para uma
--		determinada configurao,  necessrio guardar as mensagens ge-
--		radas, para que sejam posteriormente impressas. Esta funo guar
--		da as mensagens de erro geradas para os campos de um formulrio
--		em uma tabela auxiliar, apagando-as da tabela cgi, o que serve
--		como inicializao para o prximo teste.
-- Retorno:	retorna uma tabela com as mensagens geradas para os campos do 
--		formulrio. Cada mensagem  indexada por uma string no formato
--		msg_<identificador>, onde identificador  o membro 
--		identificador do campo associado  mensagem.

-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
   	local i, msgs, campos, id_campo
	msgs = {}
	campos = self.campos
	i = 1
	while( campos[i] ) do
		id_campo = campos[i].identificador
		msgs['msg_'..id_campo] = cgi['msg_'..id_campo]
		cgi['msg_'..id_campo] = nil
		i = i + 1 
	end
	
	return msgs
end

--------------------------------------------------------------------------------
function Formulario:imprime()
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para 
--		a exibio dos atributos especficos de um objeto da classe 
--		Formulario.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	write('<HR size = "1"><B>Campos do Formulrio</B><HR size = "1">')
   	local campos, i
	campos = self.campos
	i = 1
	while( campos[i] ) do
		campos[i]:imprime()
		write('<HR size = "1">')
		i = i + 1
	end
end

--------------------------------------------------------------------------------
function Formulario:imprimeResultadoTeste( msgs_corretivas, msgs_diagnostico )
--------------------------------------------------------------------------------
-- Descrio: 	Gera para a sada padro (disco ou browser), tags em html para 
--		a exibio dos resultados de um teste, na forma de uma tabela 
--		que relaciona nomes de campos, com valores e mensagens geradas
--		nas duas configuraes possveis: mensagens Corretivas e de
--		Diagnstico.
-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
   	local campos, i 
	campos = self.campos
	i = 1
	while( campos[i] ) do
		--if 	not campos[i].componentes then
			write('<B>Campo '..campos[i].nome..'</B> (classe Campo'..
			strupper(strsub(campos[i].tipo,1,1))..
			strsub(campos[i].tipo,2)..')<BR>\n')
			write('<TABLE BORDER = "1" CELLSPACING = "0">\n')
			write('<TR>\n<TD ALIGN = "center"><B>Valor</B></TD>\n')
			write('<TD ALIGN = "center"><B>Conf.</B></TD>\n')
			write('<TD ALIGN = "center"><B>Mensagem</B></TD>\n</TR>\n')
			write('<TR>\n<TD ALIGN = "center" ROWSPAN = "2">'..tostring(campos[i].valor)..
			'</TD>\n')
			write('<TD ALIGN = "center">C</TD>\n')		
			write('<TD>'..
			formata_msg(msgs_corretivas['msg_'..campos[i].identificador])
			..'</TD>\n</TR>\n')
			write('<TR><TD ALIGN = "center">D</TD>\n')		
			write('<TD>'..
			formata_msg(msgs_diagnostico['msg_'..campos[i].identificador])
			..'</TD>\n</TR>')
			write('</TABLE>')
		--end
		i = i + 1
	end
end

--------------------------------------------------------------------------------
function Formulario:testa(testes, prefixo_descricao)
--------------------------------------------------------------------------------
-- Descrio:	Executa um conjunto de testes dos campos do formulrio, gerando 
--		para a sada padro (browser ou disco) tags html para a exibio 
--		das condies e resultados do teste. 
-- Parmetros:	testes: tabela com os casos de teste. Esta tabela possui os se-
--		guintes ndices:
--		descricao: string contendo uma descrio para o caso de teste, 
--		pode ser nil. Neste caso no  gerada tag para a exibio da 
--		descrio.	
--		valores: tabela contendo uma tabela com os valores simulados pa-
--		ra cada campo do formulrio. Cada valor  indexado pela string 
--		que identifica o componente (seu membro identificador).

--		prefixo_descricao: string a ser exibida como prefixo da	des-
--		crio do teste. Por exemplo se se deseja gerar a exibio de
--		um conjunto de testes denominados "Teste i", passaramos 
--		"Teste: " neste parmetro. Se na verdade estamos gerando a exi-
--		bio de casos de testes, poderamos passar "Caso 1.". 
--		As descries geradas incializariam-se com "Caso 1.1", 
--		"Caso 1.2", etc.

-- Autora: 	Gilda Dahis
-- Data: 	23/07/1999
-- Alterado em:  
--------------------------------------------------------------------------------
	local i = 1
	while 	testes[i] do
		--Atribui aos campos os valores do caso de teste
		self:atribuiValores(testes[i].valores)
		write('<HR size = "1"><B>'..prefixo_descricao..i..': '..testes[i].descricao..'</B><HR size = 1>\n')	
		--Testa a crtica e gerao de mensagens com configurao 'C'		
		cgi.configuracao = 'C'
		chdir(cgilua.script_pdir..CAMINHOS[cgilua.script_vdir])
		dofile('configura_msgs.lua')
		self:critica()
		msgs_corretivas = self:guardaMsgs()
		cgi.configuracao = 'D'
		dofile( 'configura_msgs.lua')
		chdir(cgilua.script_pdir)
		self:atribuiValores(testes[i].valores)
		self:critica()
		msgs_diagnostico = self:guardaMsgs()
		self:imprimeResultadoTeste( msgs_corretivas, msgs_diagnostico )
		i =  i +  1
	end
end
