----------------------------- object.lua ---------------------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	object.lua
-- Descrio:  	Biblioteca Lua contendo funes para a criao de objetos, que
--		permitem a implementao de classes de objetos com suporte a
--		herana. 
--		Define a classe Object que  a classe bsica, ansestral de 
--		todas as outras classes que possam vir a ser definidas.
--		 
-- Data:	06/01/1998
-- Autor:      	Maria Julia Dias de Lima
--
-- 
-- Alteraes: 
--     Quem      |   Quando   |   Motivo
-- --------------|------------|-------------------------------------------------
--   Gilda Dahis   21/05/1999   criao do mtodo print() e inclusao do membro
--		 |	      |	ultmoIndice na classe Object. 			
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function Inherit (object, field)
--------------------------------------------------------------------------------
-- Descrio: 	Funo que possibilita a execuo de herana em lua. As 
--		estruturas de classes e objetos em lua so representada a partir 
--		de tabelas. Os membros e os mtodos so na verdade, elementos 
--		das tabelas que representam objetos e das classes respectiva-
--		mente. Assim, a invocao de um mtodo por um objeto  na verda-
--		de uma indexao do mtodo na tabela de sua classe. 
--		A herana consiste em buscar o mtodo nas tabelas que represen-
--		tam as superclasses, at que ele seja encontrado. Assim, a im-
--		plementao de herana foi feita da seguinte forma: quando uma 
--		indexao retorna nil, pode ser especificado uma funo para ser 
--		automaticamente chamada (tag method). Assim, foi especificada 
--		uma funo que faz a mesma indexao na tabela da classe 
--		imediatamente superior, ate que nao haja mais classe superior 
--		(quando o campo parent = nil). Com isto,  possvel indexar 
--		mtodos das superclasses de objetos. 
  
-- Parmetros:	object: tabela que foi indexada (representa a classe 
--		indexada) 
--		field: ndice procurado e no encontrado na tabela object.
-- Retorno: 	Retorna o ponteiro para o mtodo associado a indexao se ela 
--		for encontrada em alguma superclasse ou nil em caso contrrio.
-- Autora: 	Maria Julia Dias de Lima
-- Data: 	06/01/1998
-- Alterado em:	21/05/1999 por Gilda Dahis (incluso do atributo ultimoIndice)
--------------------------------------------------------------------------------
   if field == "parent" then     -- avoid loops
      return nil
   end
   local p = object.parent       -- access parent object
   if type(p) == "table" then    -- check if parent is a table
      return p[field]             -- (this may call Inherit again)
   else
      return nil
   end
end

--Associa a funo Inherit a indexao de um ndice no existente para a 
--implementao da herana
--OldIndex = setfallback("index",Inherit)
settagmethod (tag({}), "index", Inherit)


--------------------------------------------------------------------------------
-- Definio da classe Object
--------------------------------------------------------------------------------
Object = { parent = nil, ultimoIndice = 0 }
--------------------------------------------------------------------------------
--O membro da classe ultimoIndice, guarda o nmero de parmetros da classe. 
--Ele  utilizado por todos os construtores das subclasses da classe Object para 
--identificar a partir de que ndice da tabela recebida como parmetro comeam 
--os parmetros especifcos da classe, para que possam atribuir corretamente os 
--valores dos argumentos aos membros especficos da classe. 
--------------------------------------------------------------------------------
-- Mtodos da classe Object
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--Construtor de objetos da classe Object
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function Object:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe Object.
-- Parmetros:	o: tabela com os parametros para a criao do objeto.
-- Retorno: 	Retorna o objeto da classe Formulario criado.
-- Autora: 	Maria Julia Dias de Lima
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
   --print('Entrando em Object:new()<BR>' )	
   --Criao do Objeto
   p = {}
   --Um ponteiro para a classe Object  armazenado no campo Parent do
   --objeto, indicando que esta  a sua superclasse
   p.parent = self
   --print('Saindo de Object:new()<BR>' )	
   return p
end

--------------------------------------------------------------------------------
function Object:subclass (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor de classes subclasses da classe Object.
-- Parmetros:	o: tabela com os parametros para a criao do objeto classe, 
--		onde: o[1]: argumento a ser armazenado no membro ultimoIndice. 
-- Retorno: 	Retorna a nova subclasse da classe Object criada.
-- Autora: 	Maria Julia Dias de Lima
-- Data: 	06/01/1998
-- Alterado em:	21/05/1999 por Gilda Dahis (incluso do membro ultimoIndice)
--------------------------------------------------------------------------------
--Criao de uma subcalsse da classe Object
--------------------------------------------------------------------------------
   --A superclasse da classe criada  a prpria classe Object
   o.parent = self
   --Guardar o ndice do ltimo membro da classe, para que os construtores dos 
   --objetos da subclasse, determinem em que ndice da tabela 'o' recebida como 
   --parmetro comeam os seus parmetros especficos 
   o.ultimoIndice = self.ultimoIndice + o.numCampos
   
   --Para debugar
   --print( 'subclass():', 'superclasse.ultimoIndice:', self.ultimoIndice, 
   --'classe atual.ultimoIndice', o.ultimoIndice, '<BR>') 
   return o
end

--------------------------------------------------------------------------------
-- Mtodos dos objetos da classe Object
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function Object:print ()
--------------------------------------------------------------------------------
-- Descrio: 	Imprime os valores de todos os membros do objeto.
-- Autora: 	Gilda
-- Data: 	21/05/1998
--------------------------------------------------------------------------------
   print('------------------------------------------------------<BR>')
   local field, value
   field, value = next( self, nil )
   while field do
   	print(field, '=', value, '<BR>')
   	field, value = next( self, field )
   end
   print('------------------------------------------------------<BR>')
end


--------------------------------------------------------------------------------
function Object:inicializaMembro ( tabela_parametros, indice, tipo, classe )
--------------------------------------------------------------------------------
-- Descrio: 	Inicializa um membro, a partir de u
-- Autora: 	Gilda
-- Data: 	07/08/1999
--------------------------------------------------------------------------------
	if 	type(tabela_parametros[indice]) ~= tipo then
		error('Erro na construo do objeto da classe '..
		classe..': o '..indice..ordinal('o')..
		' parmetro deve ser do tipo "'..tipo..'". Consulte o manual.') 
	else
		return tabela_parametros[indice]
	end
end


