------------------------- gerador_msg_erro_formulario.lua ----------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	gerador_msg_erro.lua
-- Descrio:  	Mdulo gerador das mensagens associadas a erros de preenchimento
--		de campos em formulrios html. Carrega todas as funes necess-
--		rias para a gerao das mensagens.
		 
-- Data:	21/04/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes:
--     Quem    |   Quando   |   Motivo
-- ------------|------------|---------------------------------------------------
--   
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--Carrega os mtodos geradores de mensagens de erro
--------------------------------------------------------------------------------
--Mtodos para a gerao de partes comuns as mensagens de ao corretiva e de
--diagnstico
dofile( "funcoes_geracao.lua" )

--Mtodos para a gerao de mensagens de ao corretiva
dofile( "msg_acao_corretiva.lua" )

--Mtodos para a gerao de partes comuns as mensagens de diagnstico
dofile( "msg_diagnostico.lua" )

------------------------------------------------------------------------------
--Tabelas que associam identificadores de erro a uma configurao de mensagem
--'C' = 'Ao Corretiva' e 'D' = 'Diagnstico'
------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--Erros associados a campos da classe CampoLimitadoInterface
-------------------------------------------------------------------------------
CampoLimitadoInterface.configuracaoMsgErro = { campo_obrigatorio_vazio = 'C' }


-------------------------------------------------------------------------------
--Erros associados a campos da classe CampoIlimitado
-------------------------------------------------------------------------------
CampoIlimitado.configuracaoMsgErro = { 	restricao_tamanho_insatisfeita = 'C' }

-------------------------------------------------------------------------------
--Erros associados a campos da classe CampoSimples
-------------------------------------------------------------------------------
CampoSimples.configuracaoMsgErro = 
{ 
	campo_obrigatorio_vazio = 'C',
	tipo_invalido = 'C',
	limite_ultrapassado = 'C',
	restricao_tamanho_insatisfeita = 'D',
	valor_invalido = 'D',
	chave_ja_cadastrada = 'D'
} 

-------------------------------------------------------------------------------
--Erros associados a campos da classe CampoData
-------------------------------------------------------------------------------
CampoData.configuracaoMsgErro = CampoSimples.configuracaoMsgErro

CampoData.configuracaoMsgErro.dia_invalido = 'D'
CampoData.configuracaoMsgErro.mes_invalido = 'D'
CampoData.configuracaoMsgErro.dia_fevereiro_acima_limite = 'D'
CampoData.configuracaoMsgErro.dia_maior_que_30 ='D'
CampoData.configuracaoMsgErro.data_incompleta = 'C'

-------------------------------------------------------------------------------
--Erros associados a campos da classe CampoEstruturado
-------------------------------------------------------------------------------
CampoEstruturado.configuracaoMsgErro = 
{
	--Esta primitiva  usada quando se quer retornar uma mensagem de erro vazia para 
	--campos estruturados, em casos onde no ser gerada mensagem global, porm mensa
	--gens locais foram geradas
	erro_campo_estruturado = nil,
	restricao_tamanho_insatisfeita = 'D',
	componentes_invalidos = 'C',
	restricao_tamanho_insatisfeita_componentes_invalidos = 'C'
}

--------------------------------------------------------------------------------
function Campo:geraMensagem( id_erro, parametros )
--------------------------------------------------------------------------------
-- Descrio:	Gera as tags html para a exibio da mensagem de erro associada
--		ao campo.
--		Executa um mtodo gerador de mensagem associado a um determinado 
-- 		identificador de erro, passando para ele os parmetros necess-
--		rios. Para cada identificador de erro, h uma entrada na tabela 
--		'self.configuracaoMsgErro', que contm o tipo de mensagem de 
--		erro a ser gerada ('D' - diagnstico ou 'C' - Ao Corretiva). 
--		A funo chama o mtodo gerador de mensagem, de acordo com a 
--		configurao especificada para o erro nesta tabela. Os mtodos 
--		geradores de mensagens do tipo 'D' esto em 
--		"msg_diagnostico.lua" e os mtodos geradores de mensagens do 
--		tipo 'C', esto em "msg_acao_corretiva.lua".
-- Parmetros:	
--		id_erro:	string representando o identificador do erro
--				encontrado.
--		parametros: 	tabela com os parmetros da funo geradora da
--				mensagem.

-- Retorno: 	Retorna a string com a mensagem de erro gerada pela funo 
--		chamada. 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	-- Variveis
	------------------------------------------------------------------------
	local configuracao 	--auxiliar para armazenar o tipo de mensagem a  
				--ser gerada 
	local  msg		--auxiliar para armazenar a mensagem gerada
	local classe_msg 	--auxiliar para montar a tag '<P CLASS ...' com
				--estilo especificado no arquivo de  configura-
				--o do gerador (config.lua)
	
	if ESTILO_MSG then
		classe_msg = ' CLASS = "'..ESTILO_MSG..'"'
	else
		classe_msg = ""
	end
	msg = self:msgErro(id_erro, parametros)
	--print(self.nome, id_erro, msg,'<BR>')
	return '<P'..classe_msg..'>'..tostring(msg)..'</P>'
end

--------------------------------------------------------------------------------
function CampoLimitadoInterface:msgErro( id_erro, parametros )
--------------------------------------------------------------------------------
-- Descrio:	Gera as tags html para a exibio da mensagem de erro associada
--		ao campo.
--		Executa um mtodo gerador de mensagem associado a um determinado 
-- 		identificador de erro, passando para ele os parmetros necess-
--		rios. Para cada identificador de erro, h uma entrada na tabela 
--		'self.msgErro', que contm o tipo de mensagem de erro a ser ge-
--		rada ('D' - diagnstico ou 'C' - Ao Corretiva). A funo chama 
--		o mtodo gerador de mensagem, de acordo com a configurao espe-
--		cificada para o erro nesta tabela. Os mtodos geradores de men-
--		sagens do tipo 'D' esto em "msg_diagnostico.lua" e os mtodos 
--		geradores de mensagens do tipo 'C', esto em 
--		"msg_acao_corretiva.lua".
-- Parmetros:	
--		id_erro:	string representando o identificador do erro
--				encontrado.
--		parametros: 	tabela com os parmetros da funo geradora da
--				mensagem.

-- Retorno: 	Retorna a string com a mensagem de erro gerada pela funo 
--		chamada. 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	--PROVISRIO: ainda no descobri como chamar uma funo da superclasse
	--quando existe uma funo com o mesmo nome na subclasse. Atualmente
	--o parmetro 'self' no est sendo implicitamente passado.
	--if 	not self then
	--	self = objeto
	--end
	
	local configuracao = self.configuracaoMsgErro[ id_erro ]
	if 	id_erro == 'campo_obrigatorio_vazio' then 
		if 	configuracao == 'D' then
			return self:msgCampoObrigatorioVazio()
		elseif	configuracao == 'C' then
			return self:msgPreencherCampo() 
		end 
	end
end

--------------------------------------------------------------------------------
function CampoIlimitado:msgErro( id_erro, parametros, objeto )
--------------------------------------------------------------------------------
	--PROVISRIO: ainda no descobri como chamar uma funo da superclasse
	--quando existe uma funo com o mesmo nome na subclasse. Atualmente
	--o parmetro 'self' no est sendo implicitamente passado.
	if 	self == CampoIlimitado then
		self = objeto
	end
	local configuracao = self.configuracaoMsgErro[ id_erro ]
	--print( 'Configuracao: ', configuracao )
	
	if 	id_erro == 'restricao_tamanho_insatisfeita' then
		if 	configuracao == 'D' then
			return self:msgRestricaoTamanhoInsatisfeita( parametros )
		elseif 	configuracao == 'C' then
			return self:msgSatisfazerRestricaoTamanho( parametros )
		end
	end
end

-------------------------------------------------
--Mtodos da classe CampoSimples
-------------------------------------------------
function CampoSimples:msgErro( id_erro, parametros, objeto )
  --PROVISRIO: ainda no descobri como chamar uma funo da superclasse
	--quando existe uma funo com o mesmo nome na subclasse. Atualmente
	--o parmetro 'self' no est sendo implicitamente passado.
	if 	self == CampoSimples then
		self = objeto
	end
	--print('CampoSimples:msgErro()<BR>')
	--self:print()
	local msg = CampoIlimitado:msgErro( id_erro, parametros, self )
	if 	msg then 
		return msg
	end
		
	local configuracao = self.configuracaoMsgErro[ id_erro ]
	if 	id_erro == 'campo_obrigatorio_vazio' then 
		if 	configuracao == 'D' then
			return self:msgCampoObrigatorioVazio()
		elseif	configuracao == 'C' then
			return self:msgPreencherCampo() 
		end 
	elseif id_erro == 'tipo_invalido' then 
	   	if 	configuracao == 'D'  then
	   	 	return self:msgTipoInvalido() 
	   	elseif configuracao == 'C' then
	   		return self:msgAcertarTipo() 
		end
	elseif	id_erro == 'limite_ultrapassado' then
		if 	configuracao == 'D' then
			return self:msgLimiteUltrapassado( parametros ) 
		elseif configuracao == 'C' then
			return self:msgRespeitarLimite( parametros ) 
		end
	elseif	id_erro == 'valor_invalido' then
		if 	configuracao == 'D' then
			return self:msgValorInvalido()
		elseif configuracao == 'C' then
			return nil
		end
	elseif	id_erro == 'chave_ja_cadastrada' then
		if 	configuracao =='D' then
			return self:msgChaveJaCadastrada()
		--elseif configuracao == 'C' then
		--	return self:msgSelecionarNaLista()
		end
	end
end

-------------------------------------------------
--Mtodos da classe CampoData
-------------------------------------------------
function CampoData:msgErro( id_erro, parametros )
	local configuracao = self.configuracaoMsgErro[ id_erro ]
	local msg = CampoSimples:msgErro( id_erro, parametros, self )
	if 	msg then 
		return msg
	end
	if 	id_erro  == 'dia_invalido' then
		if 	configuracao == 'D' then
			return self:msgDiaInvalido()
		elseif configuracao == 'C' then
			return nil
		end
	elseif id_erro == 'mes_invalido' then
		if 	configuracao == 'D' then 
			return self:msgMesInvalido()
		elseif configuracao == 'C' then
			return nil
		end
	elseif id_erro == 'dia_fevereiro_acima_limite' then
		if 	configuracao == 'D' then
			return self:msgDiaFevereiroAcimaLimite(parametros)
		elseif 	configuracao == 'C' then
			return nil
		end
	elseif id_erro == 'dia_maior_que_30' then
		if 	configuracao == 'D' then
			return self:msgDiaMaiorQue30( parametros )
		elseif configuracao == 'C' then
			return nil
		end
	elseif id_erro == 'data_incompleta' then 
		if 	configuracao == 'D' then 
			return self:msgDataIncompleta()
		elseif configuracao == 'C' then
			return self:msgCompletarData()
		end
		
	end
end
-------------------------------------------------
--Mtodos da classe CampoEstruturado
-------------------------------------------------
function CampoEstruturado:msgErro( id_erro, parametros )
	local configuracao = self.configuracaoMsgErro[ id_erro ]
	local msg = CampoIlimitado:msgErro( id_erro, parametros, self )

	if 	msg then 
		return msg
	end

	if 	id_erro == 'erro_campo_estruturado' then
		--Esta primitiva  usada quando se quer retornar uma mensagem de 
		--erro vazia para campos estruturados, em casos onde no ser 
		--gerada mensagem global, porm mensagens locais foram geradas
		return ""
	elseif 	id_erro == 'componentes_invalidos' then
		if	configuracao == 'D' then
			return self:msgComponentesInvalidos()
		elseif	configuracao == 'C' then
			return self:msgCorrigirComponentes()
		end
	elseif 	
	 id_erro == 'restricao_tamanho_insatisfeita_componentes_invalidos' then
		if 	configuracao == 'D' then
			return self:msgRestricaoTamanhoInsatisfeitaComponentesInvalidos( parametros )
		elseif 	configuracao == 'C' then
			return self:msgSatisfazerRestricaoTamanhoCorrigirComponentes( parametros )
		end
	end
end	
-------------------------------------------------
--Mtodos da classe CampoEstruturado
-------------------------------------------------
--------------------------------------------------------------------------------
function Formulario:msgErro()
--------------------------------------------------------------------------------
-- Descrio: 	Retorna uma mensagem geral que indica a ocorrncia de erro no 
--		formulrio.
-- Retorno: 	Retorna uma mensagem geral que indica a ocorrncia de erro no 
--		formulrio.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local  msg		--auxiliar para armazenar a mensagem gerada
	local classe_msg 	--auxiliar para montar a tag '<P CLASS ...' com
				--estilo especificado no arquivo de  configura-
				--o do gerador (config.lua)

	msg = [[O formulrio foi preenchido incorretamente. 
		Siga as instrues junto aos campos que devem ser revistos.]]
	
	if ESTILO_MSG then
		classe_msg = ' CLASS = "'..ESTILO_MSG..'"'
	else
		classe_msg = ""
	end
	--print(self.nome, id_erro, msg,'<BR>')
	return '<P'..classe_msg..'>'..msg..'</P>'
end

